@@ -24,6 +24,7 @@
 #include "bftextview2_scanner.h"
 #include "bftextview2_identifier.h"
 #include "bftextview2_autocomp.h"
+#include "stringlist.h"
 
 /*#undef DBG_AUTOCOMP
 #define DBG_AUTOCOMP g_print*/
@@ -40,6 +41,7 @@ typedef struct {
 	gint listwidth;
 	gint w;
 	gint h;
+	gboolean in_fill; /* TRUE while filling the liststore */
 	guint16 contextnum;
 } Tacwin;
 
@@ -148,7 +150,7 @@ acwin_check_keypress(BluefishTextView * btv, GdkEventKey * event)
 					if (pattern_id) {
 						GSList *tmpslist =
 							g_array_index(master->bflang->st->matches, Tpattern, pattern_id).autocomp_items;
-						/* a pattern MAY have multiple autocomplete items. This code is not efficient iof in the future some 
+						/* a pattern MAY have multiple autocomplete items. This code is not efficient iof in the future some
 						   patterns would have many autocomplete items. I don't expect this, so I leave this as it is right now  */
 						while (tmpslist) {
 							Tpattern_autocomplete *pac = tmpslist->data;
@@ -219,6 +221,10 @@ acw_selection_changed_lcb(GtkTreeSelection * selection, Tacwin * acw)
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	BluefishTextView *master = BLUEFISH_TEXT_VIEW(acw->btv->master);
+	DBG_AUTOCOMP("acw_selection_changed_lcb, in_fill=%d\n",acw->in_fill);
+	if (acw->in_fill)
+		return;
+
 	if (!g_array_index(master->bflang->st->contexts, Tcontext, acw->contextnum).patternhash
 		|| !main_v->props.show_autocomp_reference)
 		return;
@@ -231,9 +237,10 @@ acw_selection_changed_lcb(GtkTreeSelection * selection, Tacwin * acw)
 				GPOINTER_TO_INT(g_hash_table_lookup
 								(g_array_index
 								 (master->bflang->st->contexts, Tcontext, acw->contextnum).patternhash, key));
+			g_free(key);
 			if (pattern_id && g_array_index(master->bflang->st->matches, Tpattern, pattern_id).reference) {
 				GtkRequisition requisition;
-				DBG_AUTOCOMP("show %s\n",
+				DBG_AUTOCOMP("acw_selection_changed_lcb, show %s\n",
 							 g_array_index(master->bflang->st->matches, Tpattern, pattern_id).reference);
 				gtk_label_set_markup(GTK_LABEL(acw->reflabel),
 									 g_array_index(master->bflang->st->matches, Tpattern,
@@ -268,6 +275,7 @@ acwin_create(BluefishTextView * btv)
 
 	acw = g_slice_new0(Tacwin);
 	acw->btv = btv;
+	acw->in_fill = TRUE;
 	acw->win = gtk_window_new(GTK_WINDOW_POPUP);
 	gtk_widget_set_app_paintable(acw->win, TRUE);
 	gtk_window_set_resizable(GTK_WINDOW(acw->win), FALSE);
@@ -313,7 +321,7 @@ acwin_create(BluefishTextView * btv)
 	return acw;
 }
 
-/* returns TRUE if window is popped-up lower than the cursor, 
+/* returns TRUE if window is popped-up lower than the cursor,
 returns FALSE if window is popped-up higher than the cursor (because cursor is low in the screen) */
 static gboolean
 acwin_position_at_cursor(BluefishTextView * btv)
@@ -344,13 +352,14 @@ acwin_position_at_cursor(BluefishTextView * btv)
 }
 
 static void
-acwin_calculate_window_size(Tacwin * acw, GList * items, GList * items2, const gchar *closetag)
+acwin_calculate_window_size(Tacwin * acw, GList * items, GList * items2, const gchar *closetag, gint *numitems)
 {
 	GList *tmplist;
 	gboolean runtwo = FALSE;
 	gchar *longest = NULL, *tmp;
-	guint longestlen = 1, numitems = 0;
-	DBG_AUTOCOMP("acwin_calculate_window_size, items=%p, items2=%p\n", items, items2);
+	guint longestlen = 1;
+	*numitems = 0;
+	DBG_AUTOCOMP("acwin_calculate_window_size, items=%p, items2=%p, closetag=%s\n", items, items2, closetag);
 	if (closetag) {
 		longest = g_markup_escape_text(closetag, -1);
 		longestlen = strlen(longest);
@@ -364,6 +373,11 @@ acwin_calculate_window_size(Tacwin * acw, GList * items, GList * items2, const g
 			if (!tmplist)
 				break;
 		}
+		g_assert(tmplist != NULL);
+		g_assert(tmplist->data != NULL);
+		DBG_AUTOCOMP("acwin_calculate_window_size, tmplist=%p", tmplist);
+		DBG_AUTOCOMP(", tmplist->data=%p",tmplist->data);
+		DBG_AUTOCOMP("=%s\n", (gchar *)tmplist->data);
 		tmp = g_markup_escape_text(tmplist->data, -1);
 		len = strlen(tmp);
 		if (len > longestlen) {
@@ -372,7 +386,7 @@ acwin_calculate_window_size(Tacwin * acw, GList * items, GList * items2, const g
 		} else {
 			g_free(tmp);
 		}
-		numitems++;
+		(*numitems)++;
 		tmplist = g_list_next(tmplist);
 	}
 	if (longest) {
@@ -381,9 +395,9 @@ acwin_calculate_window_size(Tacwin * acw, GList * items, GList * items2, const g
 		DBG_AUTOCOMP("longest=%s\n", longest);
 		pango_layout_set_markup(panlay, longest, -1);
 		pango_layout_get_pixel_size(panlay, &len, &rowh);
-		acw->h = MIN(MAX((numitems + 1) * rowh + 8, 150), 350);
+		acw->h = MIN(MAX((*numitems + 1) * rowh + 8, 150), 350);
 		acw->w = acw->listwidth = MIN(len + 20, 350);
-		DBG_AUTOCOMP("numitems=%d, rowh=%d, new height=%d, listwidth=%d\n", numitems, rowh, acw->h,
+		DBG_AUTOCOMP("numitems=%d, rowh=%d, new height=%d, listwidth=%d\n", *numitems, rowh, acw->h,
 				acw->listwidth);
 		gtk_widget_set_size_request(GTK_WIDGET(acw->scroll), acw->listwidth, acw->h);	/* ac_window */
 		g_free(longest);
@@ -391,61 +405,65 @@ acwin_calculate_window_size(Tacwin * acw, GList * items, GList * items2, const g
 	}
 }
 
-/* not only fills the tree, but calculates and sets the required width as well */
+static int
+ac_sort_func(const gchar *a, const gchar *b)
+{
+	if (a && b && a[0] == '<' && b[0] == '<' && a[1] != b[1]) {
+		/* make sure that </ is sorted below <[a-z] */
+		if (a[1] == '/') return 1;
+		if (b[1] == '/') return -1;
+		return a[1]-b[1];
+	}
+	return g_strcmp0(a,b);
+}
+
+/* fills the tree */
 static void
-acwin_fill_tree(Tacwin * acw, GList * items, GList * items2, gchar * closetag, gboolean reverse)
+acwin_fill_tree(Tacwin * acw, GList * items, GList * items2, gchar * closetag, gboolean reverse, gint numitems)
 {
 	GList *tmplist, *list = NULL;
-	/*gchar *longest = NULL; */
-	/*guint numitems = 0, longestlen = 1; */
+	gint everynth=1, i=0;
+
+	/* to avoid that we show 10000 items in the autocomplete list, we insert every N items where N % everynth == 0 */
+	if (numitems > 512) {
+		everynth = 1.0 * numitems / 512.0;
+		DBG_AUTOCOMP("numitems=%d, everynth=%d\n", numitems, everynth);
+	}
 	if (items)
 		list = g_list_copy(items);
-	/*g_print("got %d items\n",g_list_length(items));
-	   g_print("got %d items2\n",g_list_length(items2)); */
 	if (items2)
 		list = g_list_concat(g_list_copy(items2), list);
-	/*g_print("got %d list\n",g_list_length(list)); */
 
-	list = g_list_sort(list, (GCompareFunc) g_strcmp0);
-	if (closetag)
-		list = g_list_prepend(list, closetag);
+	list = g_list_sort(list, (GCompareFunc) ac_sort_func);
+	if (closetag) {
+		GList *tlist2;
+		tlist2 = find_in_stringlist(list, closetag);
+		if (tlist2) {
+			list = g_list_remove_link(list, tlist2);
+			list = g_list_concat(tlist2, list);
+		} else {
+			list = g_list_prepend(list, closetag);
+		}
+	}
 	if (reverse) {
 		list = g_list_reverse(list);
 		DBG_AUTOCOMP("reverse list!\n");
 	}
 	tmplist = g_list_first(list);
-	while (tmplist /* && numitems < 50 */ ) {
-		GtkTreeIter it;
-		gchar *tmp;
-		/*guint len; */
-		gtk_list_store_append(acw->store, &it);
-		tmp = g_markup_escape_text(tmplist->data, -1);
-/*		len = strlen(tmplist->data);
-		if (len > longestlen) {
-			g_free(longest);
-			longest = tmp;
-			longestlen = len;
-		}*/
-		gtk_list_store_set(acw->store, &it, 0, tmp, 1, tmplist->data, -1);
-		DBG_AUTOCOMP("acwin_fill_tree, add item %s\n",tmp);
-/*		if (tmp != longest)*/
-		g_free(tmp);
-		/*numitems++; */
+	while (tmplist) {
+		if (i % everynth == 0) {
+			GtkTreeIter it;
+			gchar *tmp;
+			gtk_list_store_append(acw->store, &it);
+			tmp = g_markup_escape_text(tmplist->data, -1);
+			gtk_list_store_set(acw->store, &it, 0, tmp, 1, tmplist->data, -1);
+			DBG_AUTOCOMP("acwin_fill_tree, add item %s\n",tmp);
+			g_free(tmp);
+		}
+		i++;
 		tmplist = g_list_next(tmplist);
 	}
 	g_list_free(list);
-/*	if (longest) {
-		gint len, rowh;
-		PangoLayout *panlay = gtk_widget_create_pango_layout(GTK_WIDGET(acw->tree), NULL);
-		pango_layout_set_markup(panlay, longest, -1);
-		pango_layout_get_pixel_size(panlay, &len, &rowh);
-		acw->h = MIN(MAX((numitems + 1) * rowh + 8, 150), 350);
-		DBG_AUTOCOMP("numitems=%d, rowh=%d, new height=%d\n", numitems, rowh, acw->h);
-		acw->w = acw->listwidth = MIN(len + 20, 350);
-		gtk_widget_set_size_request(GTK_WIDGET(acw->tree), acw->listwidth, acw->h);
-		g_free(longest);
-		g_object_unref(G_OBJECT(panlay));
-	}*/
 }
 
 /* static void print_ac_items(GCompletion *gc) {
@@ -468,7 +486,6 @@ void
 autocomp_run(BluefishTextView * btv, gboolean user_requested)
 {
 	GtkTextIter cursorpos, iter;
-	GtkTextBuffer *buffer;
 	BluefishTextView *master = BLUEFISH_TEXT_VIEW(btv->master);
 	gint contextnum;
 	gunichar uc;
@@ -477,8 +494,7 @@ autocomp_run(BluefishTextView * btv, gboolean user_requested)
 	if (G_UNLIKELY(!master->bflang || !master->bflang->st))
 		return;
 
-	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(btv));
-	gtk_text_buffer_get_iter_at_mark(buffer, &cursorpos, gtk_text_buffer_get_insert(buffer));
+	gtk_text_buffer_get_iter_at_mark(btv->buffer, &cursorpos, gtk_text_buffer_get_insert(btv->buffer));
 
 	iter = cursorpos;
 	gtk_text_iter_set_line_offset(&iter, 0);
@@ -507,11 +523,13 @@ autocomp_run(BluefishTextView * btv, gboolean user_requested)
 		if (found) {
 			fblock =
 				found->numblockchange < 0 ? pop_blocks(found->numblockchange, found->fblock) : found->fblock;
-			DBG_AUTOCOMP("blockstack has pattern %d on top, with tagclose_from_blockstack=%d\n",
-						 fblock->patternum, g_array_index(btv->bflang->st->matches, Tpattern,
+			if (fblock) {
+				DBG_AUTOCOMP("blockstack has pattern %d on top, with tagclose_from_blockstack=%d\n",
+						 fblock->patternum, g_array_index(master->bflang->st->matches, Tpattern,
 														  fblock->patternum).tagclose_from_blockstack);
+			}
 /*		if (g_array_index(btv->bflang->st->matches, Tpattern, fblock->patternum).tagclose_from_blockstack) {
-				gchar *start; 
+				gchar *start;
 				gtk_text_buffer_get_iter_at_mark(buffer, &it1, fblock->start1);
 				gtk_text_buffer_get_iter_at_mark(buffer, &it2, fblock->end1);
 				gtk_text_iter_forward_char(&it1);
@@ -533,19 +551,19 @@ autocomp_run(BluefishTextView * btv, gboolean user_requested)
 		GList *items = NULL, *items2 = NULL;
 		/*print_ac_items(g_array_index(btv->bflang->st->contexts,Tcontext, contextnum).ac); */
 
-		prefix = gtk_text_buffer_get_text(buffer, &iter, &cursorpos, TRUE);
+		prefix = gtk_text_buffer_get_text(btv->buffer, &iter, &cursorpos, TRUE);
 
 		if (fblock) {
 			GString *tmpstr;
 			gint plen;
 			GtkTextIter it1;
-			gtk_text_buffer_get_iter_at_offset(buffer, &it1, fblock->start1_o);
+			gtk_text_buffer_get_iter_at_offset(btv->buffer, &it1, fblock->start1_o);
 			tmpstr = g_string_new("</");
 			while(gtk_text_iter_forward_char(&it1)) {
 				gunichar uc = gtk_text_iter_get_char(&it1);
 				if (!g_unichar_isalnum(uc) && uc != '_') {
 					break;
-				}	
+				}
 				g_string_append_c(tmpstr, uc);
 			}
 			g_string_append_c(tmpstr, '>');
@@ -585,10 +603,12 @@ autocomp_run(BluefishTextView * btv, gboolean user_requested)
 			GtkTreeSelection *selection;
 			GtkTreeIter it;
 			gboolean below;
+			gint numitems=0;
 			/* create the GUI */
 			if (!btv->autocomp) {
 				btv->autocomp = acwin_create(btv);
 			} else {
+				ACWIN(btv->autocomp)->in_fill=TRUE;
 				g_free(ACWIN(btv->autocomp)->prefix);
 				g_free(ACWIN(btv->autocomp)->newprefix);
 				ACWIN(btv->autocomp)->prefix = NULL;
@@ -600,9 +620,9 @@ autocomp_run(BluefishTextView * btv, gboolean user_requested)
 			if (newprefix) {
 				ACWIN(btv->autocomp)->newprefix = g_strdup(newprefix);
 			}
-			acwin_calculate_window_size(ACWIN(btv->autocomp), items, items2, closetag);
+			acwin_calculate_window_size(ACWIN(btv->autocomp), items, items2, closetag, &numitems);
 			below = acwin_position_at_cursor(btv);
-			acwin_fill_tree(ACWIN(btv->autocomp), items, items2, closetag, !below);
+			acwin_fill_tree(ACWIN(btv->autocomp), items, items2, closetag, !below, numitems);
 			gtk_widget_show(ACWIN(btv->autocomp)->win);
 			selection = gtk_tree_view_get_selection(ACWIN(btv->autocomp)->tree);
 			if (below)
@@ -617,6 +637,7 @@ autocomp_run(BluefishTextView * btv, gboolean user_requested)
 				gtk_tree_view_scroll_to_cell(ACWIN(btv->autocomp)->tree, path, NULL, FALSE, 0.0, 0.0);
 				gtk_tree_path_free(path);
 			}
+			ACWIN(btv->autocomp)->in_fill=FALSE;
 			gtk_tree_selection_select_iter(selection, &it);
 			g_free(closetag);
 		} else {
