@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * bftextview2_scanner.c
  *
- * Copyright (C) 2008,2009,2010,2011 Olivier Sessink
+ * Copyright (C) 2008,2009,2010,2011,2012,2013 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,8 @@
 /*#define DEVELOPMENT*/
 /*#define HL_PROFILING*/
 /*#define DUMP_SCANCACHE*/
+/*#define DUMP_SCANCACHE_UPDATE_OFFSET*/
+/*#define DUMP_HIGHLIGHTING*/
 
 /*#define VALGRIND_PROFILING*/
 
@@ -35,10 +37,10 @@
 #include "bftextview2_scanner.h"
 #include "bftextview2_identifier.h"
 
-/*#undef DBG_SCANCACHE
-#undef DBG_SCANNING
-#define DBG_SCANCACHE g_print
+/*#undef DBG_SCANNING
 #define DBG_SCANNING g_print*/
+/*#undef DBG_SCANCACHE
+#define DBG_SCANCACHE g_print*/
 
 /* use 
 G_SLICE=always-malloc G_DEBUG=gc-friendly valgrind --tool=memcheck --num-callers=32 src/bluefish
@@ -167,6 +169,36 @@ dump_scancache(BluefishTextView * btv)
 }
 #endif							/* DUMP_SCANCACHE */
 
+#ifdef DUMP_HIGHLIGHTING
+void
+dump_highlighting(BluefishTextView * btv)
+{
+	GtkTextIter it;
+	gboolean cont=TRUE;
+	gtk_text_buffer_get_iter_at_offset(btv->buffer,&it,0);
+	g_print("START OF HIGHLIGHTING DUMP\n");
+	while (cont) {
+		if (gtk_text_iter_toggles_tag(&it, NULL)) {
+			GSList *lst, *tmplst;
+			g_print("pos %d has tags ", gtk_text_iter_get_offset(&it));
+			tmplst = lst = gtk_text_iter_get_tags(&it);
+			while (tmplst) {
+				gchar *name=NULL;
+				g_object_get(tmplst->data, "name", &name, NULL);
+				g_print("%p(%s) ",tmplst->data, name);
+				tmplst=tmplst->next;
+			}
+			g_slist_free(lst);
+			g_print("\n");
+		}/* else {
+			g_print("pos %d\n", gtk_text_iter_get_offset(&it));
+		}*/
+		cont = gtk_text_iter_forward_char(&it);
+	}
+	g_print("END OF DUMP\n\n");
+}
+#endif
+
 Tfound *
 get_foundcache_next(BluefishTextView * btv, GSequenceIter ** siter)
 {
@@ -203,6 +235,9 @@ get_foundcache_at_offset(BluefishTextView * btv, guint offset, GSequenceIter **
 	GSequenceIter *siter;
 	Tfound fakefound;
 	Tfound *found = NULL;
+	
+	g_assert(btv == btv->master);
+	
 	fakefound.charoffset_o = offset;
 	siter = g_sequence_search(btv->scancache.foundcaches, &fakefound, foundcache_compare_charoffset_o, NULL);
 	if (G_LIKELY(!g_sequence_iter_is_begin(siter))) {
@@ -258,7 +293,7 @@ remove_cache_entry(BluefishTextView * btv, Tfound ** found, GSequenceIter ** sit
 {
 	Tfound *tmpfound1 = *found;
 	GSequenceIter *tmpsiter1 = *siter;
-	guint invalidoffset = 0;
+	guint invalidoffset;
 	gint blockstackcount = 0, contextstackcount = 0;
 	
 	if (!tmpfound1)
@@ -267,7 +302,8 @@ remove_cache_entry(BluefishTextView * btv, Tfound ** found, GSequenceIter ** sit
 	*found = get_foundcache_next(btv, siter);
 	DBG_SCANCACHE("remove_cache_entry, STARTED, remove %p at offset %d and any children, numblockchange=%d, numcontextchange=%d\n", tmpfound1,
 				  tmpfound1->charoffset_o, tmpfound1->numblockchange, tmpfound1->numcontextchange);
-		/* if this entry pops blocks or contexts, mark the ends of those as undefined */
+	invalidoffset = tmpfound1->charoffset_o;
+	/* if this entry pops blocks or contexts, mark the ends of those as undefined */
 	if (tmpfound1->numblockchange < 0) {
 		Tfoundblock *tmpfblock = tmpfound1->fblock;
 		DBG_SCANCACHE("remove_cache_entry, found %p pops blocks, mark end of %d fblock's as undefined, fblock=%p\n",
@@ -349,6 +385,9 @@ foundcache_update_offsets(BluefishTextView * btv, guint startpos, gint offset)
 	Tfound *found;
 	GSequenceIter *siter;
 	gint comparepos;
+	
+	g_assert(btv == btv->master);
+	
 	if (offset == 0)
 		return;
 	comparepos = (offset < 0) ? startpos - offset : startpos;
@@ -482,7 +521,7 @@ foundcache_update_offsets(BluefishTextView * btv, guint startpos, gint offset)
 		found->charoffset_o += offset;
 		found = get_foundcache_next(btv, &siter);
 	}
-#ifdef DUMP_SCANCACHE
+#ifdef DUMP_SCANCACHE_UPDATE_OFFSET
 	dump_scancache(btv);
 #endif
 }
@@ -513,13 +552,15 @@ found_free_lcb(gpointer data, gpointer btv)
 }
 
 static void
-remove_all_highlighting_in_area(BluefishTextView * btv, GtkTextIter * start, GtkTextIter * end)
+remove_all_highlighting_in_area(BluefishTextView * btv, GtkTextIter * start, GtkTextIter * end, guint endoffset)
 {
 	GList *tmplist = g_list_first(btv->bflang->tags);
+	DEBUG_MSG("remove_all_highlighting_in_area %d:%d\n",gtk_text_iter_get_offset(start),gtk_text_iter_get_offset(end));
 	while (tmplist) {
 		gtk_text_buffer_remove_tag(btv->buffer, (GtkTextTag *) tmplist->data, start, end);
 		tmplist = g_list_next(tmplist);
 	}
+	btv->needremovetags = endoffset;
 }
 
 /*
@@ -682,8 +723,8 @@ pop_and_apply_contexts(BluefishTextView * btv, gint numchange, Tfoundcontext * c
 	guint offset = gtk_text_iter_get_offset(matchstart);
 	Tfoundcontext *fcontext = curcontext;
 	while (num < 0 && fcontext) {	/* pop, but don't pop if there is nothing to pop (because of an error in the language file) */
-		DBG_SCANNING("pop_and_apply_contexts, end context %d at pos %d, has tag %p and parent %p\n",
-					 fcontext->context, gtk_text_iter_get_offset(matchstart),
+		DBG_SCANNING("pop_and_apply_contexts, end context %d at %d:%d, has tag %p and parent %p\n",
+					 fcontext->context, fcontext->start_o, gtk_text_iter_get_offset(matchstart),
 					 g_array_index(btv->bflang->st->contexts, Tcontext, fcontext->context).contexttag,
 					 fcontext->parentfcontext);
 		fcontext->end_o = offset;
@@ -890,7 +931,7 @@ enlarge_scanning_region_to_iter(BluefishTextView * btv, Tscanning * scanning, Gt
 {
 	if (gtk_text_iter_compare(iter, &scanning->end) > 0) {
 		DBG_SCANCACHE("enlarge_scanning_region to offset %d\n", gtk_text_iter_get_offset(iter));
-		remove_all_highlighting_in_area(btv, &scanning->end, iter);
+		remove_all_highlighting_in_area(btv, &scanning->end, iter, gtk_text_iter_get_offset(iter));
 		scanning->end = *iter;
 		return TRUE;
 	}
@@ -1184,7 +1225,7 @@ bftextview2_run_scanner(BluefishTextView * btv, GtkTextIter * visible_end)
 	GtkTextIter mstart;
 	/*GArray *matchstack; */
 	Tscanning scanning;
-	guint pos = 0, newpos, reconstruction_o;
+	guint pos = 0, newpos, reconstruction_o, endoffset;
 	gboolean end_of_region = FALSE, last_character_run = FALSE, continue_loop = TRUE, finished;
 	gint loop = 0;
 #ifdef IDENTSTORING
@@ -1238,7 +1279,7 @@ bftextview2_run_scanner(BluefishTextView * btv, GtkTextIter * visible_end)
 #ifdef VALGRIND_PROFILING
 			CALLGRIND_STOP_INSTRUMENTATION;
 #endif							/* VALGRIND_PROFILING */
-			return TRUE;
+			return TRUE; /* call us again */
 		}
 		if (gtk_text_iter_compare(&scanning.end, visible_end) > 0) {
 			DBG_DELAYSCANNING
@@ -1253,8 +1294,9 @@ bftextview2_run_scanner(BluefishTextView * btv, GtkTextIter * visible_end)
 #endif
 	iter = mstart = scanning.start;
 	if (gtk_text_iter_is_start(&scanning.start)) {
+		DBG_SCANNING("start scanning at start iter\n");
 		scanning.siter = g_sequence_get_begin_iter(btv->scancache.foundcaches);
-		scanning.nextfound = NULL;
+		scanning.nextfound = get_foundcache_first(btv, &scanning.siter);
 		scanning.curfcontext = NULL;
 		scanning.curfblock = NULL;
 		reconstruction_o = 0;
@@ -1262,7 +1304,7 @@ bftextview2_run_scanner(BluefishTextView * btv, GtkTextIter * visible_end)
 		/* reconstruct the context stack and the block stack */
 		reconstruction_o = reconstruct_scanning(btv, &iter, &scanning);
 		pos = 0;
-		DBG_SCANNING("reconstructed stacks, context=%d, startstate=%d\n", scanning.context, pos);
+		DBG_SCANNING("reconstructed stacks, context=%d, startstate=%d, nextfound=%p\n", scanning.context, pos, scanning.nextfound);
 		/* now move the start position either to the start of the line, or to the position 
 		   where the stack was reconstructed, the largest offset */
 		gtk_text_buffer_get_iter_at_offset(btv->buffer, &iter, reconstruction_o);
@@ -1276,8 +1318,9 @@ bftextview2_run_scanner(BluefishTextView * btv, GtkTextIter * visible_end)
 	}
 	if (!gtk_text_iter_is_end(&scanning.end)) {
 		/* the end position should be the largest of the end of the line and the 'end' iter */
-		gtk_text_iter_forward_to_line_end(&iter);
-		gtk_text_iter_forward_char(&iter);
+		/*gtk_text_iter_forward_to_line_end(&iter);
+		gtk_text_iter_forward_char(&iter);*/
+		gtk_text_iter_forward_line(&iter);
 		if (gtk_text_iter_compare(&iter, &scanning.end) >= 0)
 			scanning.end = iter;
 		iter = scanning.start;
@@ -1287,10 +1330,9 @@ bftextview2_run_scanner(BluefishTextView * btv, GtkTextIter * visible_end)
 #ifdef HL_PROFILING
 	stage2 = g_timer_elapsed(scanning.timer, NULL);
 #endif
-	if (btv->needremovetags) {
-		remove_all_highlighting_in_area(btv, &scanning.start, &scanning.end);
-/*		remove_old_scan_results(btv, buffer, &start);*/
-		btv->needremovetags = FALSE;
+	endoffset = gtk_text_iter_get_offset(&scanning.end);
+	if (btv->needremovetags < endoffset) {
+		remove_all_highlighting_in_area(btv, &scanning.start, &scanning.end, endoffset);
 	}
 #ifdef HL_PROFILING
 	stage3 = g_timer_elapsed(scanning.timer, NULL);
@@ -1468,6 +1510,10 @@ bftextview2_run_scanner(BluefishTextView * btv, GtkTextIter * visible_end)
 #ifdef DUMP_SCANCACHE
 	dump_scancache(btv);
 #endif
+#ifdef DUMP_HIGHLIGHTING
+	dump_highlighting(btv);
+#endif
+
 	DBG_MSG("cleaned scanning run, finished this run\n");
 	return !finished;
 }
@@ -1672,16 +1718,24 @@ scancache_check_integrity(BluefishTextView * btv, GTimer *timer) {
 		if (!found)
 			break;
 		
-		if (found->charoffset_o <= prevfound_o)
+		if (found->charoffset_o < prevfound_o) {
 			g_warning("previous found had offset %d, the next found has offset %d, not ordered correctly?!?!!\n",found->charoffset_o, prevfound_o);
+		} else if (found->charoffset_o == prevfound_o) {
+			g_warning("previous found and the next found have offset %d, duplicate!!\n",found->charoffset_o);
+		}
 		prevfound_o = found->charoffset_o;
 		if (found->numcontextchange > 0) {
 			/* push context */
 			if (found->fcontext->parentfcontext != g_queue_peek_head(&contexts)) {
-				g_warning("pushing context at %d:%d, parent contexts at %d:%d do not match!\n"
+				if (found->fcontext->parentfcontext == NULL) {
+					g_warning("pushing context at %d:%d on top of non-NULL stack, but parent contexts is NULL!?\n"
+									,found->fcontext->start_o, found->fcontext->end_o);
+				} else {
+					g_warning("pushing context at %d:%d, parent contexts at %d:%d do not match!\n"
 									,found->fcontext->start_o, found->fcontext->end_o
 									,((Tfoundcontext *)found->fcontext->parentfcontext)->start_o
 									,((Tfoundcontext *)found->fcontext->parentfcontext)->end_o);
+				}
 			}
 			g_queue_push_head(&contexts, found->fcontext);
 		} else {
