@@ -2,7 +2,7 @@
  * Bluefish HTML Editor
  * bfwin.c
  *
- * Copyright (C) 2002-2011 Olivier Sessink
+ * Copyright (C) 2002-2012 Olivier Sessink
  * Copyright (C) 2011 James Hayward
  *
  * This program is free software; you can redistribute it and/or modify
@@ -66,7 +66,7 @@ void
 bfwin_fullscreen_toggle(Tbfwin * bfwin, gboolean active)
 {
 	if (active) {
-		gtk_window_fullscreen(GTK_WINDOW(bfwin->main_window));		
+		gtk_window_fullscreen(GTK_WINDOW(bfwin->main_window));
 	} else {
 		gtk_window_unfullscreen(GTK_WINDOW(bfwin->main_window));
 	}
@@ -121,6 +121,9 @@ bfwin_notebook_switch(Tbfwin * bfwin, guint action)
 	case 6:
 		notebook_move(bfwin, FALSE);
 		break;
+	case 7:
+		g_print("bfwin_notebook_switch, should not be called for 'recent'\n");
+		break;
 	}
 }
 
@@ -379,6 +382,11 @@ bfwin_apply_session(Tbfwin * bfwin)
 
 	fb2_update_settings_from_session(bfwin);
 	bfwin_recent_menu_create(bfwin, TRUE);
+
+	if (bfwin->simplesearch_combo) {
+		combobox_empty(bfwin->simplesearch_combo);
+		combobox_fill(bfwin->simplesearch_combo, NULL, bfwin->session->searchlist);
+	}
 	/* force this session in the plugins */
 	g_slist_foreach(main_v->plugins, bfplugins_enforce_session, bfwin);
 }
@@ -432,10 +440,13 @@ bfwin_configure_event(GtkWidget * widget, GdkEvent * revent, Tbfwin * bfwin)
 				DEBUG_MSG("bfwin_configure_event, NOT-maximized, setting width=%d\n",
 						  main_v->globses.main_window_w);
 			}
-			if (main_v->props.hide_bars_on_fullscreen && (event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN)) {
+			if (event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN) {
 				gboolean fullscreen = (event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN);
-				widget_set_visible(bfwin->toolbarbox, !fullscreen);
-				widget_set_visible(gtk_widget_get_parent(bfwin->statusbar), !fullscreen);
+				if (main_v->props.hide_bars_on_fullscreen) {
+					widget_set_visible(bfwin->toolbarbox, !fullscreen);
+					widget_set_visible(gtk_widget_get_parent(bfwin->statusbar), !fullscreen);
+				}
+				sync_fullscreen_toggle(bfwin, fullscreen);
 			}
 		}
 	}
@@ -449,7 +460,7 @@ bfwin_destroy_and_cleanup(Tbfwin *bfwin)
 {
 	GList *tmplist;
 	DEBUG_MSG("bfwin_destroy_and_cleanup, started for %p\n", bfwin);
-	
+
 		/* all documents have to be freed for this window */
 	tmplist = g_list_first(bfwin->documentlist);
 	DEBUG_MSG("bfwin_cleanup, have %d documents in window %p\n", g_list_length(bfwin->documentlist),
@@ -464,7 +475,10 @@ bfwin_destroy_and_cleanup(Tbfwin *bfwin)
 	}
 
 	g_signal_handler_disconnect(bfwin->notebook, bfwin->notebook_switch_signal);
-	
+	if (bfwin->update_searchhistory_idle_id != 0) {
+		g_source_remove(bfwin->update_searchhistory_idle_id);
+	}
+
 	gtk_widget_destroy(bfwin->main_window);
 	DEBUG_MSG("bfwin_destroy_and_cleanup, main_window destroyed for bfwin %p\n", bfwin);
 
@@ -487,7 +501,7 @@ bfwin_destroy_and_cleanup(Tbfwin *bfwin)
 	if (bfwin->notebook_changed_doc_activate_id != 0) {
 		g_source_remove(bfwin->notebook_changed_doc_activate_id);
 	}
-	
+
 	DEBUG_MSG("bfwin_cleanup, unref static actiongroups\n");
 	g_object_unref(G_OBJECT(bfwin->uimanager));
 	/*g_object_unref(G_OBJECT(bfwin->globalGroup));
@@ -498,7 +512,7 @@ bfwin_destroy_and_cleanup(Tbfwin *bfwin)
 	g_object_unref(G_OBJECT(bfwin->undoGroup));
 	g_object_unref(G_OBJECT(bfwin->bookmarkGroup));
 	g_object_unref(G_OBJECT(bfwin->filebrowserGroup)); invalid unref according to valgrind */
-	
+
 	DEBUG_MSG("unref dynamic templates actiongroups\n");
 	g_object_unref(G_OBJECT(bfwin->templates_group));
 	DEBUG_MSG("unref dynamic lang_mode actiongroups\n");
@@ -530,7 +544,7 @@ bfwin_delete_event(GtkWidget * widget, GdkEvent * event, Tbfwin * bfwin)
 	/*
 	   If you return FALSE in the "delete_event" signal handler GTK will emit the "destroy" signal.
 	   Returning TRUE means you handled the event, and it should not be further propagated
-	   
+
 	   we always handle it, so we always return TRUE
 	 */
 	DEBUG_MSG("bfwin_delete_event, started for bfwin %p\n", bfwin);
@@ -538,7 +552,7 @@ bfwin_delete_event(GtkWidget * widget, GdkEvent * event, Tbfwin * bfwin)
 		bfwin_destroy_and_cleanup(bfwin);
 		return TRUE;
 	}
-	
+
 	if (have_modified_documents(bfwin->documentlist)) {
 		Tclose_mode retval = multiple_files_modified_dialog(bfwin);
 
@@ -593,6 +607,7 @@ bfwin_gotoline_from_clipboard(Tbfwin * bfwin)
 {
 	gchar *string;
 	GtkClipboard *cb;
+	DEBUG_MSG("bfwin_gotoline_from_clipboard, called!\n");
 	if (!gtk_widget_get_visible(bfwin->gotoline_frame))
 		gtk_widget_show(bfwin->gotoline_frame);
 
@@ -614,6 +629,8 @@ bfwin_gotoline_frame_show(Tbfwin * bfwin)
 	gtk_widget_grab_focus(bfwin->gotoline_entry);
 }
 
+static void simplesearch_combo_entry_activated(gpointer widget, Tbfwin * bfwin);
+
 void
 bfwin_simplesearch_show(Tbfwin *bfwin)
 {
@@ -625,13 +642,14 @@ bfwin_simplesearch_show(Tbfwin *bfwin)
 	if (gtk_text_buffer_get_selection_bounds(bfwin->current_document->buffer, &itstart, &itend)) {
 		if (gtk_text_iter_get_line(&itstart)==gtk_text_iter_get_line(&itend)) {
 			gchar *tmpstr = gtk_text_buffer_get_text(bfwin->current_document->buffer,&itstart,&itend,TRUE);
-			gtk_entry_set_text(GTK_ENTRY(bfwin->simplesearch_entry),tmpstr);
+			gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(bfwin->simplesearch_combo))),tmpstr);
 			g_free(tmpstr);
-			gtk_editable_select_region(GTK_EDITABLE(bfwin->simplesearch_entry),0,-1);
+			gtk_editable_select_region(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(bfwin->simplesearch_combo))),0,-1);
 			/* TODO: mark the current selection as the 'current' search result */
 		}
 	}
-	gtk_widget_grab_focus(bfwin->simplesearch_entry);
+	simplesearch_combo_entry_activated(bfwin->simplesearch_regex, bfwin);
+	gtk_widget_grab_focus(bfwin->simplesearch_combo);
 }
 
 void
@@ -639,18 +657,20 @@ bfwin_simplesearch_from_clipboard(Tbfwin *bfwin)
 {
 	gchar *string;
 	GtkClipboard *cb;
+	DEBUG_MSG("bfwin_simplesearch_from_clipboard, called!\n");
 	if (!gtk_widget_get_visible(bfwin->gotoline_frame))
 		gtk_widget_show(bfwin->gotoline_frame);
 
 	cb = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
 	string = gtk_clipboard_wait_for_text(cb);
 	if (string) {
-		gtk_entry_set_text(GTK_ENTRY(bfwin->simplesearch_entry),string);
-		gtk_editable_select_region(GTK_EDITABLE(bfwin->simplesearch_entry),0,-1);
+		gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(bfwin->simplesearch_combo))),string);
+		gtk_editable_select_region(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(bfwin->simplesearch_combo))),0,-1);
 		/* TODO: mark the current selection as the 'current' search result */
 		g_free(string);
 	}
-	gtk_widget_grab_focus(bfwin->simplesearch_entry);
+	simplesearch_combo_entry_activated(bfwin->simplesearch_regex, bfwin);
+	gtk_widget_grab_focus(bfwin->simplesearch_combo);
 }
 
 static void
@@ -663,7 +683,7 @@ gotoline_entry_changed(GtkEditable * editable, Tbfwin * bfwin)
 
 	if (!bfwin->current_document)
 		return;
-
+	DEBUG_MSG("gotoline_entry_changed, called!\n");
 	linestr = gtk_editable_get_chars(editable, 0, -1);
 	linenum = get_int_from_string(linestr);
 	g_free(linestr);
@@ -713,6 +733,7 @@ gotoline_entry_insert_text(GtkEditable * editable, gchar * text, gint length, gi
 void
 bfwin_gotoline_search_bar_close(Tbfwin *bfwin)
 {
+	DEBUG_MSG("bfwin_gotoline_search_bar_close, called!\n");
 	gtk_widget_hide(bfwin->gotoline_frame);
 
 	if (bfwin->simplesearch_snr3run) {
@@ -733,35 +754,64 @@ bfwin_gotoline_search_bar_close(Tbfwin *bfwin)
 static void
 gotoline_close_button_clicked(GtkButton * button, Tbfwin * bfwin)
 {
+	DEBUG_MSG("gotoline_close_button_clicked, called!\n");
 	bfwin_gotoline_search_bar_close(bfwin);
 	if (bfwin->current_document)
 		gtk_widget_grab_focus(bfwin->current_document->view);
 }
 
-static gboolean simplesearch_start(Tbfwin *bfwin, gboolean allow_single_char_search) {
+static gboolean
+simpleasearch_update_search_history(gpointer data)
+{
+	DEBUG_MSG("simpleasearch_update_search_history, called!\n");
+	if (BFWIN(data)->simplesearch_combo) {
+		const gchar *tmpstr = gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(BFWIN(data)->simplesearch_combo))));
+		DEBUG_MSG("currenbt string=%s\n",tmpstr);
+		combobox_empty(BFWIN(data)->simplesearch_combo);
+		combobox_fill(BFWIN(data)->simplesearch_combo, tmpstr, BFWIN(data)->session->searchlist);
+		BFWIN(data)->update_searchhistory_idle_id = 0;
+	}
+	return FALSE;
+}
+
+static gboolean
+simplesearch_start(Tbfwin *bfwin, gboolean allow_single_char_search) {
 	const gchar *tmpstr;
 	gboolean retval=FALSE;
 
 	if (!bfwin->current_document)
 		return FALSE;
-	tmpstr = gtk_entry_get_text(GTK_ENTRY(bfwin->simplesearch_entry));
+	tmpstr = gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(bfwin->simplesearch_combo))));
 	if (bfwin->simplesearch_snr3run) {
 		DEBUG_MSG("free simple search run %p\n", bfwin->simplesearch_snr3run);
 		snr3run_free(bfwin->simplesearch_snr3run);
 		bfwin->simplesearch_snr3run=NULL;
 	}
 	if (tmpstr && tmpstr[0]!='\0' && (allow_single_char_search || tmpstr[1] != '\0')) {
-		DEBUG_MSG("start simple search run\n");
-		bfwin->simplesearch_snr3run = simple_search_run(bfwin, tmpstr);
+		gpointer before;
+		bfwin->session->ssearch_regex = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(bfwin->simplesearch_regex));
+		bfwin->session->ssearch_casesens = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(bfwin->simplesearch_casesens));
+		bfwin->session->ssearch_dotmatchall = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(bfwin->simplesearch_dotmatchall));
+		bfwin->session->ssearch_unescape = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(bfwin->simplesearch_unescape));
+		g_free(bfwin->session->ssearch_text);
+		bfwin->session->ssearch_text = g_strdup(tmpstr);
+		DEBUG_MSG("start simple search run with %s\n",tmpstr);
+		before = bfwin->session->searchlist;
+		bfwin->simplesearch_snr3run = simple_search_run(bfwin, tmpstr
+				, bfwin->session->ssearch_regex ? snr3type_pcre :snr3type_string
+				, bfwin->session->ssearch_casesens
+				, bfwin->session->ssearch_dotmatchall
+				, bfwin->session->ssearch_unescape);
+		if (before != bfwin->session->searchlist) {
+			/* update the combo with the new history */
+			if (bfwin->update_searchhistory_idle_id) {
+				g_source_remove(bfwin->update_searchhistory_idle_id);
+			}
+			bfwin->update_searchhistory_idle_id = g_timeout_add_seconds(4, simpleasearch_update_search_history,bfwin);
+		}
 		retval = TRUE;
 	}
-	return retval;	
-}
-
-static void
-simplesearch_entry_changed_or_activate(GtkEditable * editable, Tbfwin * bfwin)
-{
-	simplesearch_start(bfwin, FALSE);
+	return retval;
 }
 
 static void
@@ -779,10 +829,37 @@ simplesearch_back_clicked(GtkButton * button, Tbfwin * bfwin)
 		return;
 	snr3_run_go(bfwin->simplesearch_snr3run, FALSE);
 }
+
+static void
+simplesearch_combo_entry_changed(gpointer widget, Tbfwin * bfwin)
+{
+	DEBUG_MSG("simplesearch_combo_entry_changed, call simplesearch_start()\n");
+	simplesearch_start(bfwin, FALSE);
+}
+
+static void
+simplesearch_combo_entry_activated(gpointer widget, Tbfwin * bfwin)
+{
+	DEBUG_MSG("simplesearch_combo_entry_activated, call simplesearch_start()\n");
+	simplesearch_forward_clicked(NULL, bfwin);
+}
+
+static void
+simplesearch_option_toggled(gpointer widget, Tbfwin * bfwin)
+{
+	DEBUG_MSG("simplesearch_option_toggled, call simplesearch_start()\n");
+	simplesearch_start(bfwin, FALSE);
+	if (widget == bfwin->simplesearch_regex) {
+		gboolean regex = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(bfwin->simplesearch_regex));
+		gtk_widget_set_sensitive(bfwin->simplesearch_dotmatchall, regex);
+		gtk_widget_set_sensitive(bfwin->simplesearch_unescape, !regex);
+	}
+}
+
 static void
 simplesearch_advanced_clicked(GtkButton * button, Tbfwin * bfwin)
 {
-	const gchar *tmpstr = gtk_entry_get_text(GTK_ENTRY(bfwin->simplesearch_entry));
+	const gchar *tmpstr = gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(bfwin->simplesearch_combo))));
 	snr3_advanced_dialog(bfwin, tmpstr?tmpstr:"");
 	gotoline_close_button_clicked(NULL, bfwin);
 }
@@ -794,12 +871,17 @@ gotoline_entries_key_press_event(GtkWidget * widget, GdkEventKey * event, Tbfwin
 		gotoline_close_button_clicked(NULL, bfwin);
 		return TRUE;
 	}
+	if (event->keyval == GDK_KEY_Return && widget == bfwin->gotoline_entry) {
+		gotoline_entry_changed(GTK_EDITABLE(widget), bfwin);
+		return TRUE;
+	}
 	return FALSE;
 }
 static void
 gotoline_frame_create(Tbfwin * bfwin)
 {
 	GtkWidget *button, *hbox;
+	GtkWidget *menu;
 
 	bfwin->gotoline_frame = gtk_frame_new(NULL);
 	gtk_frame_set_shadow_type(GTK_FRAME(bfwin->gotoline_frame), GTK_SHADOW_NONE);
@@ -823,18 +905,41 @@ gotoline_frame_create(Tbfwin * bfwin)
 	gtk_box_pack_start(GTK_BOX(hbox), gtk_vseparator_new(), FALSE, FALSE, 6);
 #endif
 	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Find:")), FALSE, FALSE, 0);
-	bfwin->simplesearch_entry = gtk_entry_new();
-	gtk_box_pack_start(GTK_BOX(hbox), bfwin->simplesearch_entry, FALSE, FALSE, 0);
-	button = dialog_button_new_with_image(NULL,GTK_STOCK_GO_BACK, G_CALLBACK(simplesearch_back_clicked), bfwin, TRUE, FALSE);
+	DEBUG_MSG("ssearch_text=%s\n",bfwin->session->ssearch_text);
+	bfwin->simplesearch_combo = combobox_with_popdown(bfwin->session->ssearch_text, bfwin->session->searchlist, TRUE);
+	gtk_box_pack_start(GTK_BOX(hbox), bfwin->simplesearch_combo, FALSE, FALSE, 0);
+	button = (GtkWidget *)gtk_tool_button_new_from_stock(GTK_STOCK_GO_BACK);
+	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(simplesearch_back_clicked), bfwin);
 	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
-	button = dialog_button_new_with_image(NULL,GTK_STOCK_GO_FORWARD, G_CALLBACK(simplesearch_forward_clicked), bfwin, TRUE, FALSE);
+	button = (GtkWidget *)gtk_menu_tool_button_new_from_stock(GTK_STOCK_GO_FORWARD);
+	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(simplesearch_forward_clicked), bfwin);
 	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
-	g_signal_connect(G_OBJECT(bfwin->simplesearch_entry), "key-press-event", G_CALLBACK(gotoline_entries_key_press_event), bfwin);
-	g_signal_connect(bfwin->simplesearch_entry, "changed", G_CALLBACK(simplesearch_entry_changed_or_activate), bfwin);
-	g_signal_connect(bfwin->simplesearch_entry, "activate", G_CALLBACK(simplesearch_entry_changed_or_activate), bfwin);
-	button = dialog_button_new_with_image(_("Advanced"), NULL, G_CALLBACK(simplesearch_advanced_clicked),bfwin, FALSE, TRUE);
+	menu = gtk_menu_new();
+	bfwin->simplesearch_regex = gtk_check_menu_item_new_with_label(_("Regular expression (pcre)"));
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(bfwin->simplesearch_regex), bfwin->session->ssearch_regex);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), GTK_WIDGET(bfwin->simplesearch_regex));
+	bfwin->simplesearch_casesens = gtk_check_menu_item_new_with_label(_("Case sensitive"));
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(bfwin->simplesearch_casesens), bfwin->session->ssearch_casesens);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), GTK_WIDGET(bfwin->simplesearch_casesens));
+	bfwin->simplesearch_dotmatchall = gtk_check_menu_item_new_with_label(_("Dot character matches newlines"));
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(bfwin->simplesearch_dotmatchall), bfwin->session->ssearch_dotmatchall);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), GTK_WIDGET(bfwin->simplesearch_dotmatchall));
+	bfwin->simplesearch_unescape = gtk_check_menu_item_new_with_label(_("Pattern contains backslash escaped sequences"));
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(bfwin->simplesearch_unescape), bfwin->session->ssearch_unescape);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), GTK_WIDGET(bfwin->simplesearch_unescape));
+	gtk_menu_tool_button_set_menu(GTK_MENU_TOOL_BUTTON(button), GTK_WIDGET(menu));
+	gtk_widget_show_all(GTK_WIDGET(menu));
+	g_signal_connect(gtk_bin_get_child(GTK_BIN(bfwin->simplesearch_combo)), "key-press-event", G_CALLBACK(gotoline_entries_key_press_event), bfwin);
+	g_signal_connect(gtk_bin_get_child(GTK_BIN(bfwin->simplesearch_combo)), "changed", G_CALLBACK(simplesearch_combo_entry_changed), bfwin);
+	g_signal_connect(gtk_bin_get_child(GTK_BIN(bfwin->simplesearch_combo)), "activate", G_CALLBACK(simplesearch_combo_entry_activated), bfwin);
+	g_signal_connect(bfwin->simplesearch_regex, "toggled", G_CALLBACK(simplesearch_option_toggled), bfwin);
+	g_signal_connect(bfwin->simplesearch_casesens, "toggled", G_CALLBACK(simplesearch_option_toggled), bfwin);
+	g_signal_connect(bfwin->simplesearch_dotmatchall, "toggled", G_CALLBACK(simplesearch_option_toggled), bfwin);
+	g_signal_connect(bfwin->simplesearch_unescape, "toggled", G_CALLBACK(simplesearch_option_toggled), bfwin);
+	button = (GtkWidget *)gtk_tool_button_new(NULL,_("Advanced"));
+	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(simplesearch_advanced_clicked),bfwin);
 	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
-	
+
 	gtk_box_pack_start(GTK_BOX(bfwin->notebook_box), bfwin->gotoline_frame, FALSE, FALSE, 2);
 	gtk_widget_show_all(hbox);
 }
@@ -1045,6 +1150,7 @@ bfwin_notebook_changed(Tbfwin * bfwin, gint newpage)
 {
 	gint cur = newpage;
 	gint doclistlen;
+	Tdocument *prev_document;
 	DEBUG_MSG
 		("bfwin_notebook_changed, doclistlen=%d, newpage=%d, notebook_curpage=%d, last_notebook_page=%d, curdoc=%p\n",
 		 g_list_length(bfwin->documentlist)
@@ -1094,9 +1200,9 @@ bfwin_notebook_changed(Tbfwin * bfwin, gint newpage)
 		/*g_print("notebook_changed, set enable_scanner to FALSE for doc %p\n",bfwin->current_document); */
 		BLUEFISH_TEXT_VIEW(bfwin->current_document->view)->enable_scanner = FALSE;
 	}
-	bfwin->prev_document = bfwin->current_document;
+	prev_document = bfwin->current_document;
 	bfwin->current_document = g_list_nth_data(bfwin->documentlist, cur);
-	bfwin_current_document_changed_notify(bfwin, bfwin->prev_document, bfwin->current_document);
+	bfwin_current_document_changed_notify(bfwin, prev_document, bfwin->current_document);
 	if (bfwin->current_document == NULL) {
 		DEBUG_MSG("bfwin_notebook_changed, WEIRD 2, doclist[%d] == NULL, RETURNING\n", cur);
 		return;
@@ -1148,6 +1254,81 @@ notebook_switch_page(GtkWidget * notebook, gpointer * page, gint page_num, Tbfwi
 	bfwin_notebook_changed(bfwin, page_num);
 }
 
+static void
+notebook_connect_signals(Tbfwin *bfwin)
+{
+	/* We have to know when the notebook changes */
+	if (bfwin->notebook_switch_signal == 0) {
+	bfwin->notebook_switch_signal =
+		g_signal_connect_after(G_OBJECT(bfwin->notebook), "switch-page", G_CALLBACK(notebook_switch_page),
+							   bfwin);
+	}
+}
+
+static void
+notebook_disconnect_signals(Tbfwin *bfwin)
+{
+	if (bfwin->notebook_switch_signal > 0) {
+		g_signal_handler_disconnect(bfwin->notebook, bfwin->notebook_switch_signal);
+		bfwin->notebook_switch_signal = 0;
+	}
+}
+
+static gboolean
+bfwin_key_press_event(GtkWidget *widget,GdkEventKey  *kevent,gpointer   user_data)
+{
+	/*g_print("bfwin_key_press_event, key=%d, state=%d\n",kevent->keyval,kevent->state);*/
+	if (kevent->keyval == GDK_KEY_Tab && (kevent->state & GDK_CONTROL_MASK)) {
+		gint i;
+		Tdocument *doc;
+		GList *tmplist;
+		DEBUG_MSG("bfwin_key_press_event, control tab pressed\n");
+		/* switch to the next recent document, without activating it */
+		i = gtk_notebook_get_current_page(GTK_NOTEBOOK(BFWIN(user_data)->notebook));
+		/* we cannot use the bfwin->current_document because if we tab multiple documents
+		with control-tab the current notebook page and the current document might be out
+		of sync */
+		doc = g_list_nth_data(BFWIN(user_data)->documentlist, i);
+		if (!doc)
+			return FALSE;
+		tmplist = g_list_next(doc->recentpos);
+		if (!tmplist)
+			return FALSE;
+		i = g_list_index(BFWIN(user_data)->documentlist, tmplist->data);
+		if (i==-1)
+			return FALSE;
+		DEBUG_MSG("bfwin_key_press_event, set notebook page %d\n", i);
+		notebook_disconnect_signals(BFWIN(user_data));
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(BFWIN(user_data)->notebook), i);
+		return TRUE;
+	}
+	/*if (kevent->keyval == GDK_KEY_Control_L || kevent->keyval == GDK_KEY_Control_R) {
+		g_print("control pressed\n");
+	}*/
+	return FALSE;
+}
+
+static gboolean
+bfwin_key_release_event(GtkWidget *widget, GdkEventKey  *kevent, gpointer   user_data)
+{
+	/*g_print("bfwin_key_release_event, key=%d, state=%d\n",kevent->keyval,kevent->state);*/
+	/*if (kevent->keyval == GDK_KEY_Tab && kevent->state & GDK_CONTROL_MASK) {
+		g_print("control tab released\n");
+	}*/
+	if (kevent->keyval == GDK_KEY_Control_L || kevent->keyval == GDK_KEY_Control_R) {
+		/* if we did switch to a recent document without activating it, we should activate it now */
+		DEBUG_MSG("bfwin_key_release_event, control released\n");
+		if (BFWIN(user_data)->notebook_switch_signal == 0) {
+			gint i;
+			notebook_connect_signals(BFWIN(user_data));
+			i = gtk_notebook_get_current_page(GTK_NOTEBOOK(BFWIN(user_data)->notebook));
+			DEBUG_MSG("bfwin_key_release_event, connect signals, and call bfwin_notebook_changed(%p, %d)\n",user_data,i);
+			bfwin_notebook_changed(BFWIN(user_data), i);
+		}
+	}
+	return FALSE;
+}
+
 void
 bfwin_create_main(Tbfwin * bfwin)
 {
@@ -1178,6 +1359,8 @@ bfwin_create_main(Tbfwin * bfwin)
 					 G_CALLBACK(bfwin_configure_event), bfwin);
 	g_signal_connect(G_OBJECT(bfwin->main_window), "window-state-event",
 					 G_CALLBACK(bfwin_configure_event), bfwin);
+	g_signal_connect(G_OBJECT(bfwin->main_window), "key-press-event", G_CALLBACK(bfwin_key_press_event), bfwin);
+	g_signal_connect(G_OBJECT(bfwin->main_window), "key-release-event", G_CALLBACK(bfwin_key_release_event), bfwin);
 
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(bfwin->main_window), vbox);
@@ -1189,7 +1372,9 @@ bfwin_create_main(Tbfwin * bfwin)
 
 	/* first a menubar */
 	bfwin->uimanager = gtk_ui_manager_new();
+#if !GTK_CHECK_VERSION(3,4,0)
 	gtk_ui_manager_set_add_tearoffs(bfwin->uimanager, TRUE);
+#endif
 	bfwin_main_ui_init(bfwin, bfwin->toolbarbox);
 
 	/* then the toolbars */
@@ -1224,8 +1409,6 @@ bfwin_create_main(Tbfwin * bfwin)
 #if !GTK_CHECK_VERSION(3, 0, 0)
 	gtk_notebook_set_tab_border(GTK_NOTEBOOK(bfwin->notebook), 0);
 #endif
-	/*g_object_set(G_OBJECT(bfwin->notebook), "tab-fill", 0, NULL);
-	g_object_set(G_OBJECT(bfwin->notebook), "tab-curvature", 0, NULL);*/
 	gtk_notebook_popup_enable(GTK_NOTEBOOK(bfwin->notebook));
 
 	/* Add notebook and its fake friend to their common hbox. */
@@ -1267,18 +1450,14 @@ bfwin_create_main(Tbfwin * bfwin)
 #else
 		gtk_statusbar_set_has_resize_grip(GTK_STATUSBAR(bfwin->statusbar_lncol), FALSE);
 		gtk_statusbar_set_has_resize_grip(GTK_STATUSBAR(bfwin->statusbar_insovr), FALSE);
-		gtk_statusbar_set_has_resize_grip(GTK_STATUSBAR(bfwin->statusbar), FALSE);				
+		gtk_statusbar_set_has_resize_grip(GTK_STATUSBAR(bfwin->statusbar), FALSE);
 #endif /* gtk3 */
 		if (bfwin->session->view_statusbar)
 			gtk_widget_show_all(hbox);
 		else
 			gtk_widget_show(hbox);
 	}
-
-	/* We have to know when the notebook changes */
-	bfwin->notebook_switch_signal =
-		g_signal_connect_after(G_OBJECT(bfwin->notebook), "switch-page", G_CALLBACK(notebook_switch_page),
-							   bfwin);
+	notebook_connect_signals(bfwin);
 	g_signal_connect(G_OBJECT(bfwin->notebook), "page-reordered", G_CALLBACK(notebook_reordered), bfwin);
 
 	if (main_v->props.switch_tabs_by_altx) {
@@ -1353,7 +1532,7 @@ bfwin_show_main(Tbfwin * bfwin)
 
 	menuitem = gtk_ui_manager_get_widget(bfwin->uimanager, "/MainMenu/HelpMenu/HelpAbout");
 	gtk_osxapplication_insert_app_menu_item(theApp, menuitem,5);
-	
+
 	menuitem = gtk_ui_manager_get_widget(bfwin->uimanager, "/MainMenu/FileMenu/FileQuit");
 	gtk_widget_hide(menuitem);
 
