@@ -310,6 +310,14 @@ ui_paste(GtkAction * action, gpointer user_data)
 		doc_paste(bfwin);
 }
 
+static void
+ui_paste_special(GtkAction * action, gpointer user_data)
+{
+	Tbfwin *bfwin = BFWIN(user_data);
+	if (bfwin->current_document)
+		doc_paste_special(bfwin);
+}
+
 static void
 ui_find(GtkAction * action, gpointer user_data)
 {
@@ -333,7 +341,7 @@ ui_find_from_clipboard(GtkAction * action, gpointer user_data)
 static void
 ui_replace(GtkAction * action, gpointer user_data)
 {
-	
+
 	snr3_advanced_dialog(BFWIN(user_data), NULL);
 	/*snr_dialog_new(BFWIN(user_data), BF_REPLACE_DIALOG);*/
 }
@@ -409,7 +417,16 @@ ui_file_rename(GtkAction * action, gpointer user_data)
 	Tbfwin *bfwin = BFWIN(user_data);
 
 	if (bfwin->current_document)
-		doc_save_backend(bfwin->current_document, TRUE, TRUE, FALSE, FALSE);
+		doc_save_backend(bfwin->current_document, docsave_move, FALSE, FALSE);
+}
+
+static void
+ui_file_save_copy(GtkAction * action, gpointer user_data)
+{
+	Tbfwin *bfwin = BFWIN(user_data);
+
+	if (bfwin->current_document)
+		doc_save_backend(bfwin->current_document, docsave_copy, FALSE, FALSE);
 }
 
 static void
@@ -418,7 +435,7 @@ ui_file_save(GtkAction * action, gpointer user_data)
 	Tbfwin *bfwin = BFWIN(user_data);
 
 	if (bfwin->current_document)
-		doc_save_backend(bfwin->current_document, FALSE, FALSE, FALSE, FALSE);
+		doc_save_backend(bfwin->current_document, docsave_normal, FALSE, FALSE);
 }
 
 static void
@@ -433,7 +450,7 @@ ui_file_save_as(GtkAction * action, gpointer user_data)
 	Tbfwin *bfwin = BFWIN(user_data);
 
 	if (bfwin->current_document)
-		doc_save_backend(bfwin->current_document, TRUE, FALSE, FALSE, FALSE);
+		doc_save_backend(bfwin->current_document, docsave_saveas, FALSE, FALSE);
 }
 
 static void
@@ -525,6 +542,12 @@ ui_bookmark_previous(GtkAction * action, gpointer user_data)
 	bookmark_navigate(BFWIN(user_data), 2);
 }
 
+static void
+ui_doc_recent(GtkAction * action, gpointer user_data)
+{
+	bfwin_notebook_switch(BFWIN(user_data), 7);
+}
+
 static void
 ui_doc_first(GtkAction * action, gpointer user_data)
 {
@@ -697,6 +720,13 @@ ui_lorem_ipsum(GtkAction * action, gpointer user_data)
 	lorem_ipsum_dialog(BFWIN(user_data));
 }
 
+static void
+ui_jsbeautify(GtkAction * action, gpointer user_data)
+{
+	jsbeautify_dialog(BFWIN(user_data));
+}
+
+
 static void
 ui_insert_relative_filename(GtkAction * action, gpointer user_data)
 {
@@ -711,17 +741,22 @@ ui_insert_absolute_filename(GtkAction * action, gpointer user_data)
 
 /* view action callbacks */
 
+void
+sync_fullscreen_toggle(Tbfwin *bfwin, gboolean is_fullscreen)
+{
+	GtkAction *action = gtk_ui_manager_get_action(bfwin->uimanager, "/MainToolbar/ViewFullScreen");
+	/* TODO: This only works when "use-action-appearance" is TRUE. TRUE is default */
+	gtk_action_set_stock_id(action, (is_fullscreen ? GTK_STOCK_LEAVE_FULLSCREEN : GTK_STOCK_FULLSCREEN));
+	gtk_action_set_tooltip(action, (is_fullscreen ? _("Leave Fullscreen") : _("Fullscreen")));
+	if (gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action)) != is_fullscreen) {
+		gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action), is_fullscreen);
+	}
+}
+
 static void
 ui_fullscreen_toggle(GtkAction * action, gpointer user_data)
 {
-	gboolean active = gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action));
-
-	/* TODO: This only works when "use-action-appearance" is TRUE. TRUE is default */
-#if GTK_CHECK_VERSION(2,16,0)
-	gtk_action_set_stock_id(action, (active ? GTK_STOCK_LEAVE_FULLSCREEN : GTK_STOCK_FULLSCREEN));
-	gtk_action_set_tooltip(action, (active ? _("Leave Fullscreen") : _("Fullscreen")));
-#endif
-	bfwin_fullscreen_toggle(BFWIN(user_data), active);
+	bfwin_fullscreen_toggle(BFWIN(user_data), gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action)));
 }
 
 static void
@@ -787,9 +822,9 @@ static const GtkActionEntry top_level_menus[] = {
 	{"ToolsCommands", NULL, N_("_Commands")},
 	{"ToolsFilters", NULL, N_("_Filters")},
 	{"ToolsOutputBox", NULL, N_("_Outputbox")},
-	{"ToolsFormat", NULL, N_("Fo_rmat")},
+	{"ToolsFormat", NULL, N_("For_mat")},
 	{"ToolsInsert", NULL, N_("_Insert")},
-	{"ToolsConvert", NULL, N_("Co_nvert")}
+	{"ToolsConvert", NULL, N_("Con_vert")}
 };
 
 static const GtkActionEntry global_actions[] = {
@@ -803,9 +838,7 @@ static const GtkActionEntry global_actions[] = {
 	{"FileOpenSelection", NULL, N_("Open _from Selection"), NULL, N_("Open from Selection"),
 	 G_CALLBACK(ui_open_from_selection)},
 	{"FileInsert", NULL, N_("_Insert..."), NULL, N_("Insert file"), G_CALLBACK(ui_insert_doc)},
-	{"FilePrint", NULL, N_("_Print"), NULL, N_("Print"), G_CALLBACK(ui_print_doc)},
-	{"FileUploadDownload", NULL, N_("U_pload / Download..."), NULL, NULL,
-	 G_CALLBACK(ui_upload_download_dialog)},
+	{"FilePrint", NULL, N_("_Print..."), NULL, N_("Print"), G_CALLBACK(ui_print_doc)},
 	{"FileQuit", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q", N_("Quit Bluefish"), G_CALLBACK(ui_quit)},
 	{"EditPreferences", GTK_STOCK_PREFERENCES, N_("Preference_s..."), NULL, N_("Edit Preferences"),
 	 G_CALLBACK(ui_preferences)},
@@ -839,15 +872,15 @@ static const GtkActionEntry global_actions[] = {
 	{"ProjectOpen", NULL, N_("_Open Project..."), NULL, N_("Open project"), G_CALLBACK(ui_project_open)},
 	{"SynchTextBlock", NULL, N_("_Synchronize Text Block"), NULL, N_("Synchronize text block"),
 	 G_CALLBACK(ui_synch_text_block)},
-	{"ToggleComment", NULL, N_("To_ggle Comment"), "<shift><control>C", N_("Toggle comment"),
+	{"ToggleComment", NULL, N_("_Toggle Comment"), "<shift><control>C", N_("Toggle comment"),
 	 G_CALLBACK(ui_toggle_comment)},
 	 {"SelectBlock", NULL, N_("Select _Block"), "<shift><control>b", N_("Select block, use multiple times to select parent blocks "),
 	 G_CALLBACK(ui_select_block)},
-	 {"DuplicateLine", NULL, N_("Duplica_te Line"), "<control>d", N_("Duplicate the current line"),
+	 {"DuplicateLine", NULL, N_("D_uplicate Line"), "<control>d", N_("Duplicate the current line"),
 	 G_CALLBACK(ui_duplicate_line)},
-	 {"DeleteLine", NULL, N_("Delete Line"), "<control>y", N_("Delete the current line"),
+	 {"DeleteLine", NULL, N_("_Delete Line"), "<control>y", N_("Delete the current line"),
 	 G_CALLBACK(ui_delete_line)},
-	{"WordCount", NULL, N_("Word Cou_nt"), NULL, N_("Word count"), G_CALLBACK(ui_word_count)},
+	{"WordCount", NULL, N_("_Word Count"), NULL, N_("Word count"), G_CALLBACK(ui_word_count)},
 	{"LoremIpsum", NULL, N_("Lorem Ipsum generator"), NULL, N_("Lorem Ipsum generator"), G_CALLBACK(ui_lorem_ipsum)},
 	{"RelativeFilename", NULL, N_("Insert Relative Filename"), NULL, N_("Insert Relative Filename"), G_CALLBACK(ui_insert_relative_filename)},
 	{"AbsoluteFilename", NULL, N_("Insert Absolute Filename"), NULL, N_("Insert Absolute Filename"), G_CALLBACK(ui_insert_absolute_filename)},
@@ -864,6 +897,7 @@ static const GtkActionEntry global_actions[] = {
 	{"RewrapLines", NULL, N_("Rewrap _Lines"), NULL, N_("Rewrap lines"), G_CALLBACK(ui_rewrap_lines)},
 	{"StripTrailingWhitespace", NULL, N_("Strip T_railing Whitespace"), NULL, N_("Strip trailing whitespace"),
 	 G_CALLBACK(ui_strip_trailing_whitespace)},
+	{"JsBeautify", NULL, N_("Javascript beautifier"), NULL, N_("Javascript beautifier"), G_CALLBACK(ui_jsbeautify)},
 	 {"BrowserPreview", BF_STOCK_BROWSER_PREVIEW, N_("Preview in browser"), NULL, N_("Preview in browser"),
 	 G_CALLBACK(ui_browser_preview)}
 };
@@ -909,6 +943,7 @@ static const GtkActionEntry document_actions[] = {
 	{"FileSaveAll", NULL, N_("Sav_e All"), "<alt><shift>s", N_("Save all files"), G_CALLBACK(ui_file_save_all)},
 	{"FileRevert", GTK_STOCK_REVERT_TO_SAVED, N_("Rever_t to Saved"), NULL, N_("Reload current file"),
 	 G_CALLBACK(ui_file_reload)},
+	{"FileSaveCopy", NULL, N_("Save a cop_y..."), "<shift><control>y", N_("Save a copy"), G_CALLBACK(ui_file_save_copy)},
 	{"FileRename", NULL, N_("Rena_me..."), "F2", N_("Rename file"), G_CALLBACK(ui_file_rename)},
 	{"EditIndent", GTK_STOCK_INDENT, N_("_Indent"), "<control>period", N_("Indent"), G_CALLBACK(ui_indent)},
 	{"EditUnindent", GTK_STOCK_UNINDENT, N_("Unin_dent"), "<control>comma", N_("Unindent"),
@@ -925,6 +960,8 @@ static const GtkActionEntry document_actions[] = {
 	 G_CALLBACK(ui_move_tab_left)},
 	{"DocMoveRight", NULL, N_("Move Tab _Right"), NULL, N_("Move current tab right"),
 	 G_CALLBACK(ui_move_tab_right)},
+	{"DocRecent", NULL, N_("_Recent Document"), "<control>Tab", N_("Goto most recent document"),
+	 G_CALLBACK(ui_doc_recent)},
 	{"DocFirst", GTK_STOCK_GOTO_FIRST, N_("_First Document"), "<shift><control>Page_Up", N_("Goto first document"),
 	 G_CALLBACK(ui_doc_first)},
 	{"DocLast", GTK_STOCK_GOTO_LAST, N_("_Last Document"), "<shift><control>Page_Down", N_("Goto last document"),
@@ -933,7 +970,7 @@ static const GtkActionEntry document_actions[] = {
 	 G_CALLBACK(ui_doc_next)},
 	{"DocPrevious", GTK_STOCK_GO_BACK, N_("_Previous Document"), "<control>Page_Up", N_("Goto previous document"),
 	 G_CALLBACK(ui_doc_previous)},
-	{"GotoLine", NULL, N_("Goto _Line"), "<control>L", N_("Goto line"), G_CALLBACK(ui_goto_line)},
+	{"GotoLine", NULL, N_("_Goto Line"), "<control>L", N_("Goto line"), G_CALLBACK(ui_goto_line)},
 	{"GotoLineSelection", NULL, N_("Goto Line Number from _Clipboard"), "<shift><control>L",
 	 N_("Goto line number in clipboard or selection"), G_CALLBACK(ui_goto_line_selection)},
 	{"JumpToReference", NULL, N_("_Jump to Reference"), "<control>J", N_("Jump to reference"),
@@ -943,7 +980,8 @@ static const GtkActionEntry document_actions[] = {
 static const GtkActionEntry edit_actions[] = {
 	{"Cut", GTK_STOCK_CUT, N_("Cu_t"), "<control>X", N_("Cut"), G_CALLBACK(ui_cut)},
 	{"Copy", GTK_STOCK_COPY, N_("_Copy"), "<control>C", N_("Copy"), G_CALLBACK(ui_copy)},
-	{"Paste", GTK_STOCK_PASTE, N_("_Paste"), "<control>V", N_("Paste"), G_CALLBACK(ui_paste)}
+	{"Paste", GTK_STOCK_PASTE, N_("_Paste"), "<control>V", N_("Paste"), G_CALLBACK(ui_paste)},
+	{"PasteSpecial", GTK_STOCK_PASTE, N_("Paste Specia_l"), "<control><shift>v", N_("Paste Special"), G_CALLBACK(ui_paste_special)},
 };
 
 static const GtkActionEntry find_replace_actions[] = {
@@ -951,7 +989,7 @@ static const GtkActionEntry find_replace_actions[] = {
 	{"FindAgain", NULL, N_("Find A_gain"), "<control>G", N_("Find again"), G_CALLBACK(ui_find_again)},
 	{"FindSelection", NULL, N_("Find from Clip_board"), "<shift><control>F", N_("Find from clipboard or selection"),
 	 G_CALLBACK(ui_find_from_clipboard)},
-	{"Replace", GTK_STOCK_FIND_AND_REPLACE, N_("Advanced Find & Re_place..."), "<control>H", N_("Advanced Find and Replace"),
+	{"Replace", GTK_STOCK_FIND_AND_REPLACE, N_("Ad_vanced Find & Replace..."), "<control>H", N_("Advanced Find and Replace"),
 	 G_CALLBACK(ui_replace)}/*,
 	{"ReplaceAgain", NULL, N_("Replace Agai_n"), "<shift><control>H", N_("Replace again"),
 	 G_CALLBACK(ui_replace_again)}*/
@@ -994,9 +1032,11 @@ lang_mode_menu_activate(GtkAction * action, gpointer user_data)
 {
 	Tbfwin *bfwin = BFWIN(user_data);
 	Tbflang *bflang = g_object_get_data(G_OBJECT(action), "bflang");
-
-	if (gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action)) && bfwin->current_document)
+	if (!bflang)
+		return;
+	if (gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action)) && bfwin->current_document) {
 		doc_set_mimetype(bfwin->current_document, bflang->mimetypes->data, NULL);
+	}
 }
 
 void
@@ -1027,9 +1067,7 @@ lang_mode_menu_create(Tbfwin * bfwin)
 			gtk_radio_action_set_group(action, group);
 			group = gtk_radio_action_get_group(action);
 			g_object_set_data(G_OBJECT(action), "bflang", (gpointer) bflang);
-	
 			g_signal_connect(G_OBJECT(action), "activate", G_CALLBACK(lang_mode_menu_activate), bfwin);
-	
 			gtk_ui_manager_add_ui(bfwin->uimanager, bfwin->lang_mode_merge_id,
 								  "/MainMenu/DocumentMenu/DocumentLangMode/LangModePlaceholder", bflang->name,
 								  bflang->name, GTK_UI_MANAGER_MENUITEM, FALSE);
@@ -1052,7 +1090,7 @@ bfwin_main_ui_init(Tbfwin * bfwin, GtkWidget * vbox)
 	manager = bfwin->uimanager;
 
 	gtk_window_add_accel_group(GTK_WINDOW(bfwin->main_window), gtk_ui_manager_get_accel_group(manager));
-	
+
 	action_group = gtk_action_group_new("topLevelMenus");
 	gtk_action_group_set_translation_domain(action_group, GETTEXT_PACKAGE);
 	gtk_action_group_add_actions(action_group, top_level_menus, G_N_ELEMENTS(top_level_menus), bfwin);
@@ -1109,17 +1147,28 @@ bfwin_main_ui_init(Tbfwin * bfwin, GtkWidget * vbox)
 		g_warning("building main menu failed: %s", error->message);
 		g_error_free(error);
 	}
+
 #ifndef WIN32
 #ifndef MAC_INTEGRATION
 	GtkAction *action = gtk_action_new("FileOpenURL", _("Open _URL..."), NULL, NULL);
 	gtk_action_group_add_action(bfwin->globalGroup, action);
-	g_signal_connect(G_OBJECT(action), "activate", G_CALLBACK(file_open_url_cb), bfwin); 
+	g_signal_connect(G_OBJECT(action), "activate", G_CALLBACK(file_open_url_cb), bfwin);
 
 	merge_id = gtk_ui_manager_new_merge_id(manager);
 	gtk_ui_manager_add_ui(manager, merge_id,
 						  "/MainMenu/FileMenu/OpenURLPlaceholder", "FileOpenURL",
 						  "FileOpenURL", GTK_UI_MANAGER_MENUITEM, TRUE);
 	g_object_unref(action);
+
+	action = gtk_action_new("FileUploadDownload", _("Upload _/ Download..."), NULL, NULL);
+	gtk_action_group_add_action(bfwin->globalGroup, action);
+	g_signal_connect(G_OBJECT(action), "activate", G_CALLBACK(ui_upload_download_dialog), bfwin);
+
+	merge_id = gtk_ui_manager_new_merge_id(manager);
+	gtk_ui_manager_add_ui(manager, merge_id,
+						  "/MainMenu/FileMenu/UploadDownloadPlaceholder", "FileUploadDownload",
+						  "FileUploadDownload", GTK_UI_MANAGER_MENUITEM, TRUE);
+	g_object_unref(action);
 #endif							/* MAC_INTEGRATION */
 #endif							/* WIN32 */
 
@@ -1148,21 +1197,31 @@ bfwin_main_ui_init(Tbfwin * bfwin, GtkWidget * vbox)
 	bfwin_recent_menu_create(bfwin, FALSE);
 
 	set_project_menu_actions(bfwin, FALSE);
-	
+
 	bfwin_set_menu_toggle_item_from_path(bfwin->uimanager, "/MainMenu/ViewMenu/ViewSidePane", bfwin->session->view_left_panel);
 	bfwin_set_menu_toggle_item_from_path(bfwin->uimanager, "/MainMenu/ViewMenu/ViewMainToolbar", bfwin->session->view_main_toolbar);
 	bfwin_set_menu_toggle_item_from_path(bfwin->uimanager, "/MainMenu/ViewMenu/ViewStatusbar", bfwin->session->view_statusbar);
 
 	/* now the toolbars */
-
+#if GTK_CHECK_VERSION(3,4,0)
+	bfwin->main_toolbar_hb = gtk_grid_new();
+	gtk_orientable_set_orientation(GTK_ORIENTABLE(bfwin->main_toolbar_hb), GTK_ORIENTATION_HORIZONTAL);
+	bfwin->html_toolbar_hb = gtk_grid_new();
+	gtk_orientable_set_orientation(GTK_ORIENTABLE(bfwin->html_toolbar_hb), GTK_ORIENTATION_HORIZONTAL);
+#else
 	bfwin->main_toolbar_hb = gtk_handle_box_new();
-	gtk_box_pack_start(GTK_BOX(bfwin->toolbarbox), bfwin->main_toolbar_hb, FALSE, FALSE, 0);
 	bfwin->html_toolbar_hb = gtk_handle_box_new();
+#endif
+	gtk_box_pack_start(GTK_BOX(bfwin->toolbarbox), bfwin->main_toolbar_hb, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(bfwin->toolbarbox), bfwin->html_toolbar_hb, FALSE, FALSE, 0);
 
 	toolbar = gtk_ui_manager_get_widget(manager, "/MainToolbar");
 	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
 	gtk_toolbar_set_icon_size(GTK_TOOLBAR(toolbar), GTK_ICON_SIZE_LARGE_TOOLBAR);
+#if GTK_CHECK_VERSION(3,4,0)
+	/* This shouldn't be necessary, but work around a sizing issue when using GtkGrid */
+	gtk_toolbar_set_show_arrow(GTK_TOOLBAR(toolbar), FALSE);
+#endif
 	gtk_container_add(GTK_CONTAINER(bfwin->main_toolbar_hb), toolbar);
 
 	gtk_widget_show(toolbar);
@@ -1278,11 +1337,11 @@ bfwin_set_menu_toggle_item_from_path(GtkUIManager * manager, const gchar * path,
 		gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action), is_active);
 }
 
-static void 
-dynamic_menu_item_create(GtkUIManager *uimanager, GtkActionGroup *action_group, 
-						guint merge_id, const gchar *path, 
-						const gchar *action_name, const gchar *action_label, 
-						gint radio_value /* -1 for non-radio-item */, GSList **radio_group,  
+static void
+dynamic_menu_item_create(GtkUIManager *uimanager, GtkActionGroup *action_group,
+						guint merge_id, const gchar *path,
+						const gchar *action_name, const gchar *action_label,
+						gint radio_value /* -1 for non-radio-item */, GSList **radio_group,
 						GCallback callback, gpointer callbackdata, gpointer actiondata)
 {
 	GtkAction *action;
@@ -1292,7 +1351,7 @@ dynamic_menu_item_create(GtkUIManager *uimanager, GtkActionGroup *action_group,
 		g_warning("duplicate action %s, ignoring\n", action_name);
 		return;
 	}
-	
+
 	if (radio_value == -1) {
 		action = gtk_action_new(action_name, action_label, NULL, NULL);
 	} else {
@@ -1312,7 +1371,7 @@ dynamic_menu_item_create(GtkUIManager *uimanager, GtkActionGroup *action_group,
 	DEBUG_MSG("dynamic_menu_item_create, add action %s at path %s\n", action_name, path);
 	gtk_ui_manager_add_ui(uimanager, merge_id,path, action_name,action_name, GTK_UI_MANAGER_MENUITEM, TRUE);
 	g_object_unref(action);
-} 
+}
 
 void
 bfwin_encoding_set_wo_activate(Tbfwin * bfwin, const gchar * encoding)
@@ -1323,20 +1382,17 @@ bfwin_encoding_set_wo_activate(Tbfwin * bfwin, const gchar * encoding)
 	gchar *temp = g_ascii_strup(encoding, -1);
 	GtkAction *action = gtk_action_group_get_action(bfwin->encodings_group, temp);
 	g_free(temp);
-	
+
 	if (!action) {
 		g_warning("Cannot set menu action encoding %s\n", encoding);
 		return;
 	}
-	
+
 	if (!gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action))) {
-#if GTK_CHECK_VERSION(2,16,0)
-		gtk_action_block_activate(action);
-#endif
+		gpointer adata = g_object_get_data(G_OBJECT(action), "adata");
+		g_object_set_data(G_OBJECT(action), "adata", NULL);
 		gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action), TRUE);
-#if GTK_CHECK_VERSION(2,16,0)
-		gtk_action_unblock_activate(action);
-#endif
+		g_object_set_data(G_OBJECT(action), "adata", adata);
 	}
 }
 
@@ -1345,6 +1401,7 @@ bfwin_lang_mode_set_wo_activate(Tbfwin * bfwin, Tbflang * bflang)
 {
 	if (bflang) {
 		GtkAction *action = gtk_action_group_get_action(bfwin->lang_mode_group, bflang->name);
+		DEBUG_MSG("bfwin_lang_mode_set_wo_activate, got action %p for bflang=%s\n",action,bflang->name);
 		if (!action) {
 			/* because we hide certain languages from the menu it is perfectly fine if we cannot find an
 			action for a certain language file. */
@@ -1352,13 +1409,11 @@ bfwin_lang_mode_set_wo_activate(Tbfwin * bfwin, Tbflang * bflang)
 		}
 
 		if (!gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action))) {
-#if GTK_CHECK_VERSION(2,16,0)
-			gtk_action_block_activate(action);
-#endif
+			/*gtk_action_block_activate(action); this only blocks a direct call to gtk_action_activate, not a set_active() */
+			g_object_set_data(G_OBJECT(action), "bflang", (gpointer) NULL);
 			gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action), TRUE);
-#if GTK_CHECK_VERSION(2,16,0)
-			gtk_action_unblock_activate(action);
-#endif
+			g_object_set_data(G_OBJECT(action), "bflang", (gpointer) bflang);
+			/*gtk_action_unblock_activate(action);*/
 		}
 	}
 }
@@ -1392,12 +1447,12 @@ bfwin_commands_menu_create(Tbfwin * bfwin)
 		 */
 		if (g_strv_length(arr) == 4 &&(arr[0][0]=='1'||arr[0][0]=='3')) {
 			if (arr[3][0] == '1') {
-				dynamic_menu_item_create(bfwin->uimanager,bfwin->commands_group, bfwin->commands_merge_id, 
-						"/MainMenu/ToolsMenu/DefaultBrowserPlaceholder", 
+				dynamic_menu_item_create(bfwin->uimanager,bfwin->commands_group, bfwin->commands_merge_id,
+						"/MainMenu/ToolsMenu/DefaultBrowserPlaceholder",
 						arr[1], arr[1], -1, NULL, G_CALLBACK(commands_menu_activate), bfwin, arr);
 			} else {
-				dynamic_menu_item_create(bfwin->uimanager,bfwin->commands_group, bfwin->commands_merge_id, 
-						"/MainMenu/ToolsMenu/ToolsCommands/CommandsPlaceholder", 
+				dynamic_menu_item_create(bfwin->uimanager,bfwin->commands_group, bfwin->commands_merge_id,
+						"/MainMenu/ToolsMenu/ToolsCommands/CommandsPlaceholder",
 						arr[1], arr[1], -1, NULL, G_CALLBACK(commands_menu_activate), bfwin, arr);
 			}
 		}
@@ -1407,27 +1462,33 @@ bfwin_commands_menu_create(Tbfwin * bfwin)
 static void
 encodings_menu_activate(GtkAction * action, gpointer user_data)
 {
-	if (gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action))) {
-		Tbfwin *bfwin = BFWIN(user_data);
-		gchar **arr = g_object_get_data(G_OBJECT(action), "adata");
-		DEBUG_MSG("encodings_menu_activate, encoding=%s\n", arr[1]);
-		if (arr[1] && bfwin->current_document) {
-			if ((!bfwin->current_document->encoding
-				 || strcmp(arr[1], bfwin->current_document->encoding) != 0)) {
-				if (bfwin->current_document->encoding)
-					g_free(bfwin->current_document->encoding);
-				bfwin->current_document->encoding = g_strdup(arr[1]);
-				doc_set_tooltip(bfwin->current_document);
-				if (main_v->props.auto_set_encoding_meta) {
-					update_encoding_meta_in_file(bfwin->current_document, bfwin->current_document->encoding);
-				}
-				DEBUG_MSG("encodings_menu_activate, set to %s\n", arr[1]);
+	Tbfwin *bfwin = BFWIN(user_data);
+	gchar **arr;
+	if (!gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action))) {
+		return;
+	}
+	arr = g_object_get_data(G_OBJECT(action), "adata");
+	if (!arr)
+		return;
+
+	DEBUG_MSG("encodings_menu_activate, encoding=%s\n", arr[1]);
+	if (arr[1] && bfwin->current_document) {
+		if ((!bfwin->current_document->encoding
+			 || strcmp(arr[1], bfwin->current_document->encoding) != 0)) {
+			if (bfwin->current_document->encoding)
+				g_free(bfwin->current_document->encoding);
+			bfwin->current_document->encoding = g_strdup(arr[1]);
+			doc_set_tooltip(bfwin->current_document);
+			doc_set_statusbar_lang_encoding(bfwin->current_document);
+			if (main_v->props.auto_set_encoding_meta) {
+				update_encoding_meta_in_file(bfwin->current_document, bfwin->current_document->encoding);
 			}
-			if (bfwin->session->encoding)
-				g_free(bfwin->session->encoding);
-			bfwin->session->encoding = g_strdup(arr[1]);
-			DEBUG_MSG("encodings menu activate, session encoding now is %s\n", bfwin->session->encoding);
+			DEBUG_MSG("encodings_menu_activate, set to %s\n", arr[1]);
 		}
+		if (bfwin->session->encoding)
+			g_free(bfwin->session->encoding);
+		bfwin->session->encoding = g_strdup(arr[1]);
+		DEBUG_MSG("encodings menu activate, session encoding now is %s\n", bfwin->session->encoding);
 	}
 }
 
@@ -1452,8 +1513,8 @@ bfwin_encodings_menu_create(Tbfwin * bfwin)
 
 		if (g_strv_length(arr) == 3 && arr[2][0] == '1') {
 			gchar *label = g_strdup_printf("%s (%s)", arr[0], arr[1]);
-			dynamic_menu_item_create(bfwin->uimanager,bfwin->encodings_group, bfwin->encodings_merge_id, 
-						"/MainMenu/DocumentMenu/DocumentEncoding/EncodingPlaceholder", 
+			dynamic_menu_item_create(bfwin->uimanager,bfwin->encodings_group, bfwin->encodings_merge_id,
+						"/MainMenu/DocumentMenu/DocumentEncoding/EncodingPlaceholder",
 						arr[1], label, value, &group, G_CALLBACK(encodings_menu_activate), bfwin, arr);
 			g_free(label);
 			value++;
@@ -1463,6 +1524,7 @@ bfwin_encodings_menu_create(Tbfwin * bfwin)
 
 typedef struct {
 	Tbfwin *bfwin;
+	GtkWidget *neveraskagain;
 	Tselectionsave *selsave;
 	const gchar *command;
 } Tfilterdialog;
@@ -1473,6 +1535,9 @@ filter_dialog_response(GtkWidget * widget, gint response_id, gpointer user_data)
 	Tfilterdialog *fd = user_data;
 	gint begin = 0, end = -1;
 
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fd->neveraskagain))) {
+		main_v->globses.filter_on_selection_mode = (response_id == 1) ? 1 : 2;
+	}
 	gtk_widget_destroy(widget);
 	doc_restore_selection(fd->selsave, TRUE);	/* the restore will also free the Tselectionsave */
 	if (response_id == 1 && fd->bfwin->current_document)
@@ -1492,25 +1557,33 @@ filters_menu_activate(GtkAction * action, gpointer user_data)
 	/* if we have a selection, and the filter can be used on a selection,
 	   we should ask if it should be the complete file or the selection */
 
-	if (operatable_on_selection(command)
-		&& (bfwin->current_document && doc_has_selection(bfwin->current_document))) {
-		GtkWidget *dialog, *button;
-		Tfilterdialog *fd;
+	if (main_v->globses.filter_on_selection_mode != 2 && operatable_on_selection(command) && bfwin->current_document && doc_has_selection(bfwin->current_document)) {
+		if (main_v->globses.filter_on_selection_mode == 1) {
+			DEBUG_MSG("filters_menu_activate, don't ask, just work on selection\n");
+			doc_get_selection(bfwin->current_document, &begin, &end);
+			filter_command(bfwin, command, begin, end);
+		} else {
+			GtkWidget *dialog, *button, *hbox;
+			Tfilterdialog *fd;
 
-		fd = g_slice_new(Tfilterdialog);
-		fd->bfwin = bfwin;
-		fd->command = command;
+			fd = g_slice_new(Tfilterdialog);
+			fd->bfwin = bfwin;
+			fd->command = command;
 
-		fd->selsave = doc_save_selection(bfwin->current_document);
-		/* TODO: this dialog is not very convenient, hitting enter should choose 'selection' */
-		dialog =
-			gtk_message_dialog_new(GTK_WINDOW(bfwin->main_window), GTK_DIALOG_DESTROY_WITH_PARENT,
-								   GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, _("Text area to filter"));
-		gtk_dialog_add_button(GTK_DIALOG(dialog), _("All text"), 0);
-		button = gtk_dialog_add_button(GTK_DIALOG(dialog), _("Selected text only"), 1);
-		gtk_widget_grab_default(button);
-		g_signal_connect(dialog, "response", G_CALLBACK(filter_dialog_response), fd);
-		gtk_widget_show_all(dialog);
+			fd->selsave = doc_save_selection(bfwin->current_document);
+			/* TODO: this dialog is not very convenient, hitting enter should choose 'selection' */
+			dialog =
+				gtk_message_dialog_new(GTK_WINDOW(bfwin->main_window), GTK_DIALOG_DESTROY_WITH_PARENT,
+									   GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, _("Text area to filter"));
+			gtk_dialog_add_button(GTK_DIALOG(dialog), _("All text"), 0);
+			button = gtk_dialog_add_button(GTK_DIALOG(dialog), _("Selected text only"), 1);
+			gtk_widget_grab_default(button);
+			fd->neveraskagain = gtk_check_button_new_with_label(_("Never ask again"));
+			hbox = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
+			gtk_box_pack_end(GTK_BOX(hbox), fd->neveraskagain, FALSE, FALSE, 4);
+			g_signal_connect(dialog, "response", G_CALLBACK(filter_dialog_response), fd);
+			gtk_widget_show_all(dialog);
+		}
 	} else {
 		DEBUG_MSG("filters_menu_activate, calling filter_command for %s\n", command);
 		filter_command(bfwin, command, begin, end);
@@ -1533,14 +1606,14 @@ bfwin_filters_menu_create(Tbfwin * bfwin)
 
 	for (list = g_list_first(main_v->props.external_filter); list; list = list->next) {
 		gchar **arr = list->data;
-		/* arr[0] = state 
+		/* arr[0] = state
 		 * arr[1] = name
 		 * arr[2] = command
 		 */
 		if (g_strv_length(arr) == 3 && (arr[0][0]=='1' || arr[0][0]=='3')) {
-			dynamic_menu_item_create(bfwin->uimanager, bfwin->filters_group, 
-						bfwin->filters_merge_id, "/MainMenu/ToolsMenu/ToolsFilters/FiltersPlaceholder", 
-						arr[1], arr[1], -1, NULL,  
+			dynamic_menu_item_create(bfwin->uimanager, bfwin->filters_group,
+						bfwin->filters_merge_id, "/MainMenu/ToolsMenu/ToolsFilters/FiltersPlaceholder",
+						arr[1], arr[1], -1, NULL,
 						G_CALLBACK(filters_menu_activate), bfwin, arr);
 		} else {
 			DEBUG_MSG("bfwin_filters_menu_create, CORRUPT ENTRY IN filter actions; array count =%d\n",
@@ -1582,9 +1655,9 @@ bfwin_outputbox_menu_create(Tbfwin * bfwin)
 		 * arr[6] = command
 		 */
 		if (g_strv_length(arr) == 7 && (arr[0][0]=='1'||arr[0][0]=='3')) {
-			dynamic_menu_item_create(bfwin->uimanager, bfwin->outputbox_group, 
-						bfwin->outputbox_merge_id, "/MainMenu/ToolsMenu/ToolsOutputBox/OutputBoxPlaceholder", 
-						arr[1], arr[1], -1, NULL,  
+			dynamic_menu_item_create(bfwin->uimanager, bfwin->outputbox_group,
+						bfwin->outputbox_merge_id, "/MainMenu/ToolsMenu/ToolsOutputBox/OutputBoxPlaceholder",
+						arr[1], arr[1], -1, NULL,
 						G_CALLBACK(outputbox_menu_activate), bfwin, arr);
 		} else {
 			DEBUG_MSG("bfwin_outputbox_menu_create, CORRUPT ENTRY IN outputbox action; array count =%d\n",
@@ -1616,12 +1689,12 @@ static void recent_menu_add(Tbfwin *bfwin, GtkMenu *menu, const gchar *curi) {
 	gtk_widget_show(menuitem);
 }
 
-static void 
+static void
 recent_menu_remove_backend(Tbfwin *bfwin, const gchar *menupath, const gchar *curi)
 {
 	GtkWidget *menuitem, *menu;
 	GList *list, *tmplist;
-	
+
 	menuitem = gtk_ui_manager_get_widget(bfwin->uimanager,menupath);
 	menu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(menuitem));
 	list = gtk_container_get_children(GTK_CONTAINER(menu));
@@ -1644,7 +1717,7 @@ bfwin_recent_menu_remove(Tbfwin *bfwin, gboolean project, const gchar *curi)
 	}
 	for (tmplist=g_list_first(main_v->bfwinlist);tmplist;tmplist=g_list_next(tmplist)) {
 		if (project || BFWIN(tmplist->data)->session == main_v->session) {
-			recent_menu_remove_backend(bfwin, 
+			recent_menu_remove_backend(bfwin,
 					project ? "/MainMenu/ProjectMenu/ProjectOpenRecent" :"/MainMenu/FileMenu/FileOpenRecent", curi);
 		}
 	}
@@ -1658,9 +1731,9 @@ recent_menu_add_backend(Tbfwin *bfwin, const gchar *menupath, const gchar *curi)
 	gint num=0;
 	menuitem = gtk_ui_manager_get_widget(bfwin->uimanager,menupath);
 	menu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(menuitem));
-	
+
 	recent_menu_add(bfwin, GTK_MENU(menu), curi);
-	
+
 	list = gtk_container_get_children(GTK_CONTAINER(menu));
 	for (tmplist = list; tmplist; tmplist = tmplist->next) {
 		if (num > main_v->props.max_recent_files) {
@@ -1669,7 +1742,7 @@ recent_menu_add_backend(Tbfwin *bfwin, const gchar *menupath, const gchar *curi)
 		num++;
 	}
 	g_list_free(list);
-	
+
 }
 
 void
@@ -1682,11 +1755,11 @@ bfwin_recent_menu_add(Tbfwin *bfwin, gboolean project, const gchar *curi)
 	}
 	for (tmplist=g_list_first(main_v->bfwinlist);tmplist;tmplist=g_list_next(tmplist)) {
 		if (project || BFWIN(tmplist->data)->session == main_v->session) {
-			recent_menu_add_backend(bfwin, 
+			recent_menu_add_backend(bfwin,
 					project ? "/MainMenu/ProjectMenu/ProjectOpenRecent" :"/MainMenu/FileMenu/FileOpenRecent", curi);
 		}
 	}
-	
+
 }
 
 static void recent_create_backend(Tbfwin *bfwin, const gchar *menupath, GList *recentlist) {
@@ -1746,13 +1819,13 @@ bfwin_templates_menu_create(Tbfwin * bfwin)
 	}
 	bfwin->templates_merge_id = gtk_ui_manager_new_merge_id(bfwin->uimanager);
 
-	for (list = g_list_last(main_v->props.templates); list; list = list->prev) {
+	for (list = g_list_last(main_v->templates); list; list = list->prev) {
 		gchar **arr = (gchar **) list->data;
 		if (arr && arr[0] && arr[1]) {
 			gchar *action_name = g_strconcat("template ", arr[0], NULL);
-			dynamic_menu_item_create(bfwin->uimanager, bfwin->templates_group, 
-						bfwin->templates_merge_id, "/MainMenu/FileMenu/NewFromTemplate/TemplatePlaceholder", 
-						action_name, arr[0], -1, NULL,  
+			dynamic_menu_item_create(bfwin->uimanager, bfwin->templates_group,
+						bfwin->templates_merge_id, "/MainMenu/FileMenu/NewFromTemplate/TemplatePlaceholder",
+						action_name, arr[0], -1, NULL,
 						G_CALLBACK(templates_menu_activate), bfwin, arr);
 			g_free(action_name);
 		}
