@@ -28,12 +28,17 @@
 #ifndef __BLUEFISH_H_
 #define __BLUEFISH_H_
 
-#define MEMORY_LEAK_DEBUG
+/*#define MEMORY_LEAK_DEBUG*/
 /*#define DEBUG_PATHS*/
 
 #define ENABLEPLUGINS
 
 #include "config.h"
+
+#ifndef DEVELOPMENT
+#define G_DISABLE_ASSERT
+#endif
+
 #define BLUEFISH_SPLASH_FILENAME PKGDATADIR"/bluefish_splash.png"
 
 #ifdef WIN32
@@ -107,8 +112,6 @@ extern void g_none(gchar * first, ...);
 #define DIRCHR '/'
 #endif							/* WIN32 */
 
-#include <sys/types.h>
-#include <regex.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -251,6 +254,7 @@ typedef struct {
 	gboolean highlightstate;	/* does this document use highlighting ? */
 	gpointer floatingview;		/* a 2nd textview widget that has its own window */
 	gpointer bfwin;
+	GList *recentpos; 	/* this points to the list element in the recent used tabs (bfwin->recentdoclist) that points to this Tdocument */
 	GtkTreeIter *bmark_parent;	/* if NULL this document doesn't have bookmarks, if
 								   it does have bookmarks they are children of this GtkTreeIter */
 } Tdocument;
@@ -300,6 +304,7 @@ typedef struct {
 	gint auto_update_meta_author;	/* auto update author meta tag on save */
 	gint auto_update_meta_date;	/* auto update date meta tag on save */
 	gint auto_update_meta_generator;	/* auto update generator meta tag on save */
+	gint strip_trailing_spaces_on_save;
 	gint encoding_search_Nbytes;	/* number of bytes to look for the encoding meta tag */
 	gint max_window_title; /* max. number of chars in the window title */
 	gint document_tabposition;
@@ -336,7 +341,9 @@ typedef struct {
 	gchar *autosave_file_prefix;
 	gchar *autosave_file_suffix;
 	gchar *language;
-	GList *templates; /* array of name:URI */
+	gint rcfile_from_old_version;
+	gint adv_textview_right_margin; /* advanced option for the textview margin */
+	gint adv_textview_left_margin; /* advanced option for the textview margin */
 } Tproperties;
 
 /* the Tglobalsession contains all settings that can change 
@@ -347,19 +354,17 @@ typedef struct {
 	gint main_window_w;			/* main window width */
 	gint two_pane_filebrowser_height;	/* position of the pane separater on the two paned file browser */
 	gint left_panel_width;		/* width of filelist */
-	/*gint lasttime_filetypes; / * see above */
-	/*gint lasttime_encodings; / * see above */
-	gint snr_select_match;		/* if the search and replace should select anything found or just mark it */
 	gint bookmarks_default_store;	/* 0= temporary by default, 1= permanent by default */
 	gint image_thumbnail_refresh_quality;	/* 1=GDK_INTERP_BILINEAR, 0=GDK_INTERP_NEAREST */
 	gint image_thumbnailsizing_type;	/* scaling ratio=0, fixed width=1, height=2, width+height (discard aspect ratio)=3 */
 	gint image_thumbnailsizing_val1;	/* the width, height or ratio, depending on the value above */
 	gint image_thumbnailsizing_val2;	/* height if the type=3 */
 	gchar *image_thumnailformatstring;	/* like <a href="%r"><img src="%t"></a> or more advanced */
+	gint filter_on_selection_mode; /* 0=ask, 1=selection, 2=text */
 	GList *filefilters;			/* filefilter.c file filtering */
 	GList *reference_files;		/* all reference files */
 	GList *recent_projects;
-	GList *encodings;			/* all encodings you can choose from */
+	GList *encodings;			/* all encodings you can choose from, array with 0=human name, 1=name, 2="0" or "1" if it should be user visible or not */
 #ifdef WITH_MSG_QUEUE
 	gint msg_queue_poll_time;	/* milliseconds, automatically tuned to your system */
 #endif
@@ -386,6 +391,11 @@ typedef struct {
 	gint snr3_escape_chars;
 	gint snr3_dotmatchall;
 	gint snr3_recursion_level;
+	/* simplesearch options */
+	gint ssearch_regex;
+	gint ssearch_dotmatchall;
+	gint ssearch_unescape;
+	gint ssearch_casesens;
 
 	gint sync_delete_deprecated;
 	gint sync_include_hidden;
@@ -408,7 +418,8 @@ typedef struct {
 	gint outputb_show_all_output;
 	gint convertcolumn_horizontally;
 	gint display_right_margin;
-	/* 39 * sizeof(gint) */
+	gint show_visible_spacing;
+	/* 44 * sizeof(gint) */
 	/* IF YOU EDIT THIS STRUCTURE PLEASE EDIT THE CODE IN PROJECT.C THAT COPIES
 	   A Tsessionvar INTO A NEW Tsessionvar AND ADJUST THE SIZES!!!!!!!!!!!!!!!!!!!!!! */
 #ifdef HAVE_LIBENCHANT
@@ -418,6 +429,7 @@ typedef struct {
 #endif
 	/* if you add strings or lists to the session, please make sure they are free'ed 
 	in free_session() in project.c */
+	gchar *ssearch_text;
 	gchar *default_mime_type;
 	gchar *template;
 	gchar *convertcolumn_separator;
@@ -458,10 +470,10 @@ typedef struct {
 typedef struct {
 	Tsessionvars *session;		/* points to the global session, or to the project session */
 	Tdocument *current_document;	/* one object out of the documentlist, the current visible document */
-	Tdocument *prev_document;
 	gboolean focus_next_new_doc;	/* for documents loading in the background, switch to the first that is finished loading */
 	gint num_docs_not_completed;	/* number of documents that are loading or closing */
-	GList *documentlist;		/* document.c and others: all Tdocument objects */
+	GList *documentlist;		/* document.c and others: all Tdocument objects in the order of the tabs */
+	GList *recentdoclist; /* all Tdocument objects with the most recently used on top, every Tdocument has a pointer to it's own list element called doc->recentpos */
 	Tdocument *last_activated_doc;
 	Tproject *project;			/* might be NULL for a default project */
 	GtkWidget *main_window;
@@ -484,8 +496,13 @@ typedef struct {
 	guint notebook_changed_doc_activate_id;
 	guint statusbar_pop_id;
 	guint notebook_switch_signal;
+	guint update_searchhistory_idle_id;
 	GtkWidget *gotoline_entry;
-	GtkWidget *simplesearch_entry;
+	GtkWidget *simplesearch_combo;
+	GtkWidget *simplesearch_regex;
+	GtkWidget *simplesearch_casesens;
+	GtkWidget *simplesearch_dotmatchall;
+	GtkWidget *simplesearch_unescape;
 	gpointer simplesearch_snr3run;
 	GtkWidget *notebook;
 	GtkWidget *notebook_fake;
@@ -558,6 +575,7 @@ typedef struct {
 	Tsessionvars *session;		/* holds all session variables for non-project windows */
 	gpointer fb2config;			/* filebrowser2config */
 	GList *filefilters;			/* initialized by fb2config functions */
+	GList *templates; 			/* loaded in rcfile.c */
 	Tdocument *bevent_doc;
 	gint bevent_charoffset; 	/* for example used in the spellcheck code to find on which 
 											word the user clicked */
@@ -585,6 +603,12 @@ extern EXPORT Tmain *main_v;
 /* public functions from bluefish.c */
 void bluefish_exit_request(void);
 
+/* Avoid lots of warnings from API depreciated in GTK 3.0. -Wdeprecated-declarations */
+#if GTK_CHECK_VERSION(3,0,0)
+#define gtk_hbox_new(arg, arg2) gtk_box_new(GTK_ORIENTATION_HORIZONTAL, arg2)
+#define gtk_vbox_new(arg, arg2) gtk_box_new(GTK_ORIENTATION_VERTICAL, arg2)
+#endif
+
 /* backwards compatibility */
 #if !GTK_CHECK_VERSION(2,24,0)
 #define GDK_KEY_Enter GDK_Enter
@@ -633,28 +657,7 @@ void bluefish_exit_request(void);
 #define gtk_combo_box_text_new gtk_combo_box_new_text
 #define gtk_combo_box_text_append_text gtk_combo_box_append_text
 #define gtk_combo_box_text_prepend_text gtk_combo_box_prepend_text
+#define gtk_combo_box_text_remove gtk_combo_box_remove_text
 #endif /* GTK_CHECK_VERSION(2,24,0) */
-#if !GTK_CHECK_VERSION(2,18,0)
-#define gtk_widget_set_can_focus(arg, arg2) do {if (arg2) GTK_WIDGET_SET_FLAGS(arg, GTK_CAN_FOCUS); else GTK_WIDGET_UNSET_FLAGS(arg, GTK_CAN_FOCUS);} while(0)
-#define gtk_widget_get_allocation(arg1, arg2) (*(arg2) = arg1->allocation)
-#define gtk_dialog_get_content_area(arg) (GTK_DIALOG(arg)->vbox)
-#define gtk_widget_get_window(arg) (GTK_WIDGET(arg)->window)
-#define gtk_widget_get_visible(arg) (GTK_WIDGET_VISIBLE(arg))
-#define gtk_widget_set_can_default(arg, arg2) do {if (arg2) GTK_WIDGET_SET_FLAGS(arg, GTK_CAN_DEFAULT); else GTK_WIDGET_UNSET_FLAGS(arg, GTK_CAN_DEFAULT);} while(0) 
-#define gtk_dialog_get_action_area(arg) (GTK_DIALOG(arg)->action_area)
-#define gtk_selection_data_get_data(arg) (arg->data)
-#define gtk_selection_data_get_length(arg) (arg->length)
-#define gtk_selection_data_get_format(arg) (arg->format)
-#define gtk_selection_data_get_target(arg) (arg->target)
-#define gtk_widget_get_state(arg) GTK_WIDGET_STATE(arg)
-#define gtk_widget_is_sensitive(arg) GTK_WIDGET_IS_SENSITIVE(arg)
-#endif /* GTK_CHECK_VERSION(2,18,0) */
-#if !GTK_CHECK_VERSION(2,16,0)
-#define gtk_menu_item_get_label(arg) (gtk_label_get_text(GTK_LABEL(gtk_bin_get_child(GTK_BIN(arg)))))
-#endif /* GTK_CHECK_VERSION(2,16,0) */
-#if !GTK_CHECK_VERSION(2,14,0)
-#define gtk_adjustment_get_upper(arg) (GTK_ADJUSTMENT(arg)->upper)
-#define gtk_adjustment_get_lower(arg) (GTK_ADJUSTMENT(arg)->lower)
-#define gtk_adjustment_get_page_size(arg) (GTK_ADJUSTMENT(arg)->page_size)
-#endif /* GTK_CHECK_VERSION(2,18,0) */
+
 #endif							/* __BLUEFISH_H_ */
