@@ -21,5 +21,10 @@
 #define __DOC_EXTERN_TOOLS_H_
 
 void lorem_ipsum_dialog(Tbfwin *bfwin);
+void jsbeautify_dialog(Tbfwin *bfwin);
+
+#ifdef WIN32
+gboolean check_python();
+#endif
 
 #endif /*__DOC_EXTERN_TOOLS_H_*/
