@@ -53,6 +53,7 @@ void doc_update_highlighting(Tbfwin * bfwin, guint callback_action, GtkWidget *
 void doc_set_wrap(Tdocument * doc, gboolean enabled);
 void doc_set_tooltip(Tdocument * doc);
 void doc_set_title(Tdocument * doc);
+void doc_set_statusbar_lang_encoding(Tdocument * doc);
 void doc_set_mimetype(Tdocument * doc, const gchar * mimetype, const gchar * filename);
 void doc_reset_filetype(Tdocument * doc, GFile * newuri, gconstpointer buf, gssize buflen);
 void doc_set_font(Tdocument * doc, gchar * fontstring);
@@ -60,7 +61,7 @@ void doc_set_tabsize(Tdocument * doc, gint tabsize);
 gint doc_get_tabsize(Tdocument * doc);
 void doc_change_tabsize(Tdocument * doc, gint direction);
 void doc_font_size(Tdocument * doc, gint direction);
-void session_set_savedir(Tbfwin * bfwin, gchar * curi);
+void doc_paste_special(Tbfwin *bfwin);
 
 gboolean doc_is_empty_non_modified_and_nameless(Tdocument * doc);
 gboolean test_docs_modified(GList * doclist);
