@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * external_commands.c - backend for external commands, filters and the outputbox
  *
- * Copyright (C) 2005-2011 Olivier Sessink
+ * Copyright (C) 2005-2013 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -53,6 +53,8 @@ typedef struct {
 	const gchar *formatstring; /* the command from the configuration, so including placeholders such as %i, %f, |,  etc. */
 	gchar *commandstring; /* the command that will be started by g_spawn, the placeholders should have been replaced/removed */
 	CustomCommandCallback customcommand_cb;
+	StatusCodeCallback statuscode_cb;
+	gpointer data;
 
 	gchar *securedir; /* if we need any local temporary files for input or output */
 	gchar *tmp_in; /* the path for the input/output temporary filename */
@@ -62,18 +64,19 @@ typedef struct {
 	gboolean pipe_out;
 
 	gboolean include_stderr; /* if stderr should be included in the output */
-	
+
 	gint standard_input; /* the pipes that will be filled by g_spawn_async_with_pipes */
-	gint standard_output; 
-	
+	gint standard_output;
+
 	GIOChannel* channel_in; /* the GIO channels for input into the filter */
 	gchar *buffer_out; /* the buffer that is the input for the filter */
 	gchar *buffer_out_position; /* a pointer inside the above buffer that points to the data that has not yet been written to the filter */
-	
+	gboolean havedata;
+
 	GIOChannel* channel_out; /* the GIO channel for output from the filter */
 	GIOFunc channel_out_lcb; /* the callback that will be registered for output data  */
 	gpointer channel_out_data; /* the corresponding callback data */
-	
+
 	GPid child_pid; /* PID of the filter */
 	guint start_command_idle_id;
 } Texternalp;
@@ -98,7 +101,7 @@ static void externalp_unref(Texternalp *ep) {
 			unlink(ep->inplace);
 			g_free(ep->inplace);
 		}
-		
+
 		if (ep->buffer_out) g_free(ep->buffer_out);
 		if (ep->securedir) {
 			rmdir(ep->securedir);
@@ -160,18 +163,21 @@ static void child_watch_lcb(GPid pid,gint status,gpointer data) {
 	Texternalp *ep = data;
 	GError *gerror=NULL;
 	DEBUG_MSG("child_watch_lcb, child exited with status=%d\n",status);
-	
+	if (ep->statuscode_cb) {
+		ep->statuscode_cb(status, BFWIN(ep->bfwin), ep->data);
+	}
 	if (ep->pipe_in && !ep->buffer_out) {
+		DEBUG_MSG("child_watch_lcb, the child has exited before we actually started\n");
 		/* the child has exited before we actually started to write data to the child, just abort now */
 		g_source_remove(ep->start_command_idle_id);
 		externalp_unref(ep); /* unref twice, once for the start_command_idle, once for child_watch_lcb */
 		externalp_unref(ep);
 		return;
 	}
-	
+
 	/* if there was a temporary output file, we should now open it and start to read it */
 	if (ep->tmp_out) {
-		
+
 		ep->channel_out = g_io_channel_new_file(ep->tmp_out,"r",&gerror);
 		DEBUG_MSG("child_watch_lcb, created channel_out from file %s\n",ep->tmp_out);
 		if (gerror) {
@@ -193,6 +199,7 @@ static void child_watch_lcb(GPid pid,gint status,gpointer data) {
 		g_io_add_watch(ep->channel_out, G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP,ep->channel_out_lcb,ep->channel_out_data);
 	}
 	g_spawn_close_pid(pid);
+	DEBUG_MSG("child_watch_lcb, almost finished, call externalp_unref\n");
 	externalp_unref(ep);
 }
 
@@ -231,12 +238,12 @@ start_command_idle(gpointer data)
 		g_io_add_watch(ep->channel_out, G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP,ep->channel_out_lcb,ep->channel_out_data);
 	}
 	externalp_unref(ep);
-	
+
 	return FALSE; /* don't call me again */
 }
 
 static void start_command_backend(Texternalp *ep) {
-#ifdef USEBINSH	
+#ifdef USEBINSH
 	gchar *argv[4];
 #else
 	gchar **argv;
@@ -255,10 +262,13 @@ static void start_command_backend(Texternalp *ep) {
     g_shell_parse_argv(ep->commandstring, NULL, &argv, NULL);
 #endif
 	DEBUG_MSG("start_command_backend,commandstring=%s\n",ep->commandstring);
-	ep->include_stderr = FALSE;
 	DEBUG_MSG("start_command_backend, pipe_in=%d, pipe_out=%d, include_stderr=%d\n",ep->pipe_in,ep->pipe_out,ep->include_stderr);
+#ifdef USEBINSH
 	DEBUG_MSG("start_command_backend, about to spawn process /bin/sh -c %s\n",argv[2]);
-	g_spawn_async_with_pipes(NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH, (ep->include_stderr)?spawn_setup_lcb:NULL, ep, &ep->child_pid, 
+#else
+	DEBUG_MSG("start_command_backend, about to spawn process %s\n",ep->commandstring);
+#endif
+	g_spawn_async_with_pipes(NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH, (ep->include_stderr)?spawn_setup_lcb:NULL, ep, &ep->child_pid,
 				(ep->pipe_in) ? &ep->standard_input : NULL,
 				(ep->pipe_out) ? &ep->standard_output : NULL,
 				NULL, &error);
@@ -277,7 +287,7 @@ static void start_command_backend(Texternalp *ep) {
 static void start_command(Texternalp *ep) {
 	if (!ep->bfwin->current_document)
 		return;
-	
+
 	if (ep->tmp_in) {
 		GError *gerror=NULL;
 		/* first create tmp_in, then start the real command in the callback */
@@ -285,7 +295,7 @@ static void start_command(Texternalp *ep) {
 		if (gerror) {
 			g_warning("failed to create input file for external program, %d: %s\n",gerror->code,gerror->message);
 			g_error_free(gerror);
-			return;	
+			return;
 		}
 		ep->buffer_out = ep->buffer_out_position = doc_get_chars(ep->bfwin->current_document,ep->begin,ep->end);
 		g_io_channel_set_flags(ep->channel_in,G_IO_FLAG_NONBLOCK,NULL);
@@ -298,11 +308,11 @@ static void start_command(Texternalp *ep) {
 
 gboolean operatable_on_selection(const gchar *formatstring) {
 
-	return (strstr(formatstring, "%f") == NULL && strstr(formatstring, "%c") == NULL 
+	return (strstr(formatstring, "%f") == NULL && strstr(formatstring, "%c") == NULL
 		&& strstr(formatstring, "%n") == NULL && strstr(formatstring, "%u") == NULL
 		&& strstr(formatstring, "%p") == NULL);
 
-} 
+}
 
 /* The format string should have new options:
     * %f local full path (function should abort for remote files)
@@ -331,7 +341,7 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 		need_filename=FALSE,
 		need_arguments=FALSE;
 	gint items = 2, cur=0;
-	
+
 	if (!ep->bfwin->current_document) {
 		return NULL;
 	}
@@ -343,7 +353,7 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 		need_pipein = TRUE;
 		formatstring[0] = ' ';
 	}
-	
+
 	if (!discard_output && (formatstring[formatstringlen-1]=='|')) {
 		need_pipeout = TRUE;
 		formatstring[formatstringlen-1]=' ';
@@ -362,25 +372,24 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 	}
 	if (ep->bfwin->current_document->uri)
 		localname = g_file_get_path(ep->bfwin->current_document->uri);
-	
+
 	if (need_local && !localname) {
 		g_free(formatstring);
 		g_free(localname);
 		return NULL;
 	}
 
-	need_tmpin = (strstr(formatstring, "%I") != NULL);
-	/* %f is for backwards compatibility with bluefish 1.0 */
-	need_tmpout = (strstr(formatstring, "%O") != NULL);
+	need_tmpin = (strstr(formatstring, "%i") != NULL);
+	need_tmpout = (strstr(formatstring, "%o") != NULL);
 	need_inplace = (strstr(formatstring, "%t") != NULL);
-	
+
 	if ((need_tmpout || need_inplace || need_pipeout) && discard_output) {
 		DEBUG_MSG("create_commandstring, external_commands should not have %%o\n");
 		g_free(formatstring);
 		g_free(localname);
 		return NULL;
 	}
-	if ((need_tmpin && (need_inplace || need_pipein)) 
+	if ((need_tmpin && (need_inplace || need_pipein))
 			|| (need_inplace && need_pipein)) {
 		DEBUG_MSG("create_commandstring, cannot have multiple inputs\n");
 		/*  BUG: give a warning that you cannot have multiple inputs */
@@ -388,7 +397,7 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 		g_free(localname);
 		return NULL;
 	}
-	if ((need_tmpout && (need_inplace || need_pipeout)) 
+	if ((need_tmpout && (need_inplace || need_pipeout))
 			|| (need_inplace && need_pipeout)) {
 		DEBUG_MSG("create_commandstring, cannot have multiple outputs, tmpout=%d, pipeout=%d, inplace=%d\n",need_tmpout,need_pipeout,need_inplace);
 		/*  BUG: give a warning that you cannot have multiple outputs */
@@ -397,7 +406,7 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 
 		return NULL;
 	}
-	
+
 	if (need_arguments) {
 		GtkWidget *dialog, *entry;
 		gchar *tmp;
@@ -426,13 +435,13 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 		}
 		gtk_widget_destroy(dialog);
 	}
-	
-	
+
+
 	if (need_pipein)  ep->pipe_in = TRUE;
 	if (need_pipeout) ep->pipe_out = TRUE;
 	DEBUG_MSG("create_commandstring, formatstring '%s' seems OK\n",formatstring);
 
-	is_local_non_modified = (ep->bfwin->current_document->uri 
+	is_local_non_modified = (ep->bfwin->current_document->uri
 		&& !ep->bfwin->current_document->modified
 		&& localname != NULL);
 
@@ -468,9 +477,9 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 		cur++;
 		if (need_preview_uri) {
 			table[cur].my_int = 'p';
-			if (ep->bfwin->session->documentroot && ep->bfwin->session->webroot 
+			if (ep->bfwin->session->documentroot && ep->bfwin->session->webroot
 					&& strlen(ep->bfwin->session->documentroot)>5
-					&& strlen(ep->bfwin->session->webroot)>5 
+					&& strlen(ep->bfwin->session->webroot)>5
 					&& strncmp(curi, ep->bfwin->session->documentroot, strlen(ep->bfwin->session->documentroot))==0) {
 						table[cur].my_char = g_strconcat(ep->bfwin->session->webroot, &curi[strlen(ep->bfwin->session->documentroot)], NULL);
 			} else {
@@ -480,7 +489,7 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 		}
 	}
 	if (need_tmpin) {
-		table[cur].my_int = 'I';
+		table[cur].my_int = 'i';
 		if (is_local_non_modified) {
 			table[cur].my_char = g_strdup(localname);
 		} else {
@@ -496,7 +505,7 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 		cur++;
 	}
 	if (need_tmpout) {
-		table[cur].my_int = 'O';
+		table[cur].my_int = 'o';
 		ep->tmp_out = create_secure_dir_return_filename();
 		table[cur].my_char = g_strdup(ep->tmp_out);
 		cur++;
@@ -535,9 +544,9 @@ static gboolean outputbox_io_watch_lcb(GIOChannel *channel,GIOCondition conditio
 		DEBUG_MSG("outputbox_io_watch_lcb, outputbox == NULL, close the channel\n");
 		g_io_channel_shutdown(channel,TRUE,NULL);
 		externalp_unref(ep);
-		return FALSE;		
+		return FALSE;
 	}
-	
+
 	if (condition & G_IO_IN) {
 		gchar *buf=NULL;
 		gsize buflen=0,termpos=0;
@@ -547,6 +556,7 @@ static gboolean outputbox_io_watch_lcb(GIOChannel *channel,GIOCondition conditio
 			if (buflen > 0) {
 				if (termpos < buflen) buf[termpos] = '\0';
 				fill_output_box(ep->bfwin->outputbox, buf);
+				ep->havedata=TRUE;
 			}
 			g_free(buf);
 			status = g_io_channel_read_line(channel,&buf,&buflen,&termpos,&gerror);
@@ -586,51 +596,32 @@ static gboolean outputbox_io_watch_lcb(GIOChannel *channel,GIOCondition conditio
 	return TRUE;
 }
 
-static gboolean filter_custom_io_watch_lcb(GIOChannel *channel,GIOCondition condition,gpointer data) {
+static gboolean full_data_io_watch_lcb(GIOChannel *channel,GIOCondition condition,gpointer data) {
 	Texternalp *ep = data;
-	DEBUG_MSG("filter_custom_io_watch_lcb, started with condition=%d\n",condition);
+	DEBUG_MSG("full_data_io_watch_lcb, started with condition=%d\n",condition);
 	if (condition & G_IO_IN) {
 		gchar *str_return;
 		gsize length;
 		GIOStatus status;
 		GError *gerror=NULL;
-		
+
 		status = g_io_channel_read_to_end(channel,&str_return,&length,&gerror);
 		if (gerror) {
 			g_warning("error while trying to read data for filter or custom command, %d: %s\n",gerror->code,gerror->message);
 			g_error_free(gerror);
 		}
-	
+
 		if (status == G_IO_STATUS_NORMAL && str_return) {
-			gint end=ep->end;
 			GError *error=NULL;
-			if (ep->customcommand_cb) {
-				ep->customcommand_cb(str_return, ep->bfwin);
-			} else {
-				GtkTextIter iter;
-				GtkTextBuffer *buffer = ep->bfwin->current_document->buffer;
-				DEBUG_MSG("filter_custom_io_watch_lcb, received '%s'\n",str_return);
-				gint line=-1,offset=-1;
-				if (ep->end == -1) {
-					end = gtk_text_buffer_get_char_count(buffer);
-				}
-				if (!gtk_text_buffer_get_has_selection(buffer)) {
-					gtk_text_buffer_get_iter_at_mark(buffer, &iter,gtk_text_buffer_get_insert(buffer));
-					line = gtk_text_iter_get_line(&iter);
-					offset = gtk_text_iter_get_line_offset(&iter);
-				}
-				doc_replace_text(ep->bfwin->current_document, str_return, ep->begin, end);
-				if (line != -1) {
-					gtk_text_buffer_get_iter_at_line_offset(buffer, &iter, line, offset);
-					gtk_text_buffer_place_cursor(buffer, &iter);
-				}
-			}
+			DEBUG_MSG("ep->customcommand_cb=%p\n",ep->customcommand_cb);
+			if (ep->customcommand_cb)
+				ep->customcommand_cb(str_return, ep->bfwin, ep->data);
 			g_io_channel_shutdown(channel,TRUE,&error);
 			externalp_unref(ep);
 			g_free(str_return);
 			return FALSE;
 		} else {
-			DEBUG_MSG("filter_custom_io_watch_lcb, status=%d\n",status);
+			DEBUG_MSG("full_data_io_watch_lcb, status=%d\n",status);
 		}
 		if (str_return) g_free(str_return);
 	}
@@ -656,68 +647,106 @@ static gboolean filter_custom_io_watch_lcb(GIOChannel *channel,GIOCondition cond
 	return TRUE;
 }
 
-typedef enum {
-	mode_filter,
-	mode_outputbox,
-	mode_command,
-	mode_custom
-} Tcommandmode;
-
-static void command_backend(Tbfwin *bfwin, gint begin, gint end, 
-								const gchar *formatstring, Tcommandmode commandmode, CustomCommandCallback customcommand_cb) {
+static void
+command_backend(Tbfwin *bfwin, gint begin, gint end,
+								const gchar *formatstring,
+								GIOFunc channel_out_cb,
+								gboolean incl_stderr,
+								StatusCodeCallback statuscode_cb,
+								CustomCommandCallback customcommand_cb, gpointer data) {
 	Texternalp *ep;
-	
+
 	DEBUG_MSG("command_backend, started\n");
 	ep = g_new0(Texternalp,1);
 	ep->bfwin = bfwin;
+	ep->customcommand_cb = customcommand_cb;
+	ep->statuscode_cb = statuscode_cb;
+	ep->data = data ? data : ep;
 	ep->doc = bfwin->current_document;
 	ep->begin = begin;
 	ep->end = end;
 	ep->formatstring = formatstring;
-	ep->commandstring = create_commandstring(ep, formatstring, (commandmode==mode_command));
+	ep->commandstring = create_commandstring(ep, formatstring, (channel_out_cb == NULL));
 	if (!ep->commandstring) {
+		gchar *tmp = g_markup_printf_escaped(_("Failed to create a command for %s."), formatstring);
+		message_dialog_new(BFWIN(ep->bfwin)->main_window, GTK_MESSAGE_ERROR
+					, GTK_BUTTONS_CLOSE, _("Failed to create a command"), tmp);
+		g_free(tmp);
 		DEBUG_MSG("command_backend, failed to create commandstring\n");
 		g_free(ep);
-		/* BUG: is the user notified of the error ?*/
 		return;
 	}
-	if (commandmode == mode_filter)	{
-		ep->include_stderr = FALSE;
-		ep->channel_out_lcb = filter_custom_io_watch_lcb;
-		ep->channel_out_data = ep;
-	} else if (commandmode == mode_outputbox)	{
-		ep->include_stderr = TRUE;
-		ep->channel_out_lcb = outputbox_io_watch_lcb;
-		ep->channel_out_data = ep;
-	} else if (commandmode == mode_command) {
-		ep->include_stderr = FALSE;
-		ep->channel_out_lcb = NULL;
-		ep->channel_out_data = NULL;
-	} else if (commandmode == mode_custom) {
-		ep->include_stderr = FALSE;
-		ep->customcommand_cb = customcommand_cb;
-		ep->channel_out_lcb = filter_custom_io_watch_lcb;
-		ep->channel_out_data = ep;
-	}
-
+	ep->channel_out_data = ep;
+	ep->include_stderr = incl_stderr;
+	ep->channel_out_lcb = channel_out_cb;
 	start_command(ep);
 }
+static void
+filter_statuscode_lcb(gint status, Tbfwin *bfwin, gpointer data)
+{
+	Texternalp *ep=data;
+	DEBUG_MSG("filter_statuscode_lcb, exitcode %d\n", status);
+#ifndef WIN32
+	/* on windows this even fires dialogs when opening something in firefox, disabling that for the moment */
+	if (status != 0) {
+		gchar *tmp = g_markup_printf_escaped(_("The command %s exited with error code %d. %s"), ep->commandstring, status, (status==32512)?_("Probably this application is not installed on your system."):"");
+		message_dialog_new(BFWIN(ep->bfwin)->main_window, GTK_MESSAGE_ERROR
+					, GTK_BUTTONS_CLOSE, _("Command returned error code"), tmp);
+		g_free(tmp);
+	}
+#endif
+}
+
+static void
+outputbox_statuscode_lcb(gint status, Tbfwin *bfwin, gpointer data)
+{
+	Texternalp *ep=data;
+	gchar *tmp = g_strdup_printf(_("The command %s exited with error code %d. %s"), ep->commandstring, status, "");
+	DEBUG_MSG("outputbox_statuscode_lcb, exitcode %d, add to outputbox: %s\n", status, tmp);
+	fill_output_box(bfwin->outputbox, tmp);
+	if (status==32512) {
+		fill_output_box(bfwin->outputbox, _("Probably this application is not installed on your system."));
+	}
+	g_free(tmp);
+}
 
 void outputbox_command(Tbfwin *bfwin, const gchar *formatstring) {
-	command_backend(bfwin, 0, -1,formatstring, mode_outputbox, NULL);
+	command_backend(bfwin, 0, -1,formatstring, outputbox_io_watch_lcb, TRUE, outputbox_statuscode_lcb,  NULL, NULL);
 }
 
+static void
+filter_custom_lcb(const gchar *str_return, gpointer bfwin, gpointer data)
+{
+	Texternalp *ep = data;
+	GtkTextIter iter;
+	GtkTextBuffer *buffer = BFWIN(bfwin)->current_document->buffer;
+	DEBUG_MSG("filter_custom_lcb, received '%s'\n",str_return);
+	gint line=-1,offset=-1, end=ep->end;
+	if (ep->end == -1) {
+		end = gtk_text_buffer_get_char_count(buffer);
+	}
+	if (!gtk_text_buffer_get_has_selection(buffer)) {
+		gtk_text_buffer_get_iter_at_mark(buffer, &iter,gtk_text_buffer_get_insert(buffer));
+		line = gtk_text_iter_get_line(&iter);
+		offset = gtk_text_iter_get_line_offset(&iter);
+	}
+	doc_replace_text(BFWIN(bfwin)->current_document, str_return, ep->begin, end);
+	if (line != -1) {
+		gtk_text_buffer_get_iter_at_line_offset(buffer, &iter, line, offset);
+		gtk_text_buffer_place_cursor(buffer, &iter);
+	}
+}
 
 void filter_command(Tbfwin *bfwin, const gchar *formatstring, gint begin, gint end) {
-	command_backend(bfwin, begin, end,formatstring, mode_filter, NULL);
+	command_backend(bfwin, begin, end,formatstring, full_data_io_watch_lcb, FALSE, filter_statuscode_lcb, filter_custom_lcb, NULL);
 }
 
 void external_command(Tbfwin *bfwin, const gchar *formatstring) {
-	command_backend(bfwin, 0, -1,formatstring, mode_command, NULL);
+	command_backend(bfwin, 0, -1,formatstring, NULL, FALSE, filter_statuscode_lcb, NULL, NULL);
 }
 
 void
-custom_command(Tbfwin *bfwin, const gchar *formatstring, CustomCommandCallback func)
+custom_command(Tbfwin *bfwin, const gchar *formatstring, CustomCommandCallback func, gpointer data)
 {
-	command_backend(bfwin, 0, -1,formatstring, mode_custom, func);
+	command_backend(bfwin, 0, -1,formatstring, full_data_io_watch_lcb, TRUE, NULL, func, data);
 }
