@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * file.c - file operations based on GIO
  *
- * Copyright (C) 2002-2011 Olivier Sessink
+ * Copyright (C) 2002-2012 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,9 +38,6 @@
 #include "stringlist.h"
 #include "async_queue.h"
 
-#if !GTK_CHECK_VERSION(2,14,0)
-#include "gtkmountoperation.h"
-#endif
 #undef OAD_MEMCOUNT
 #ifdef OAD_MEMCOUNT
 typedef struct {
@@ -88,7 +85,7 @@ file_static_queues_init(void)
 
 static GList *fileinfo_wait_for_mount = NULL;
 static GList *openfile_wait_for_mount = NULL;
-static GMountOperation *gmo = NULL;	/* we do only 1 mount operation at a 
+static GMountOperation *gmo = NULL;	/* we do only 1 mount operation at a
 									   time to avoid multiple mount calls for the same volume, resulting in multiple authentication
 									   popups for the user. In 99.9% of the cases when the user is loading multiple files that need
 									   mounting they will be from the same server anyway  */
@@ -361,6 +358,7 @@ checkNsave_replace_async_lcb(GObject * source_object, GAsyncResult * res, gpoint
 	GError *error = NULL;
 
 	g_file_replace_contents_finish(cns->uri, res, &etag, &error);
+	DEBUG_MSG("checkNsave_replace_async_lcb, finished savig to uri %p, error=%p\n",cns->uri, error);
 	if (error) {
 		DEBUG_MSG("checkNsave_replace_async_lcb,error %d: %s\n", error->code, error->message);
 		if (error->code == G_IO_ERROR_CANCELLED) {
@@ -385,7 +383,7 @@ checkNsave_replace_async_lcb(GObject * source_object, GAsyncResult * res, gpoint
 #if !GLIB_CHECK_VERSION(2, 18, 0)
 		else if (error->code == G_IO_ERROR_EXISTS
 				 && (g_file_has_uri_scheme(cns->uri, "sftp") || g_file_has_uri_scheme(cns->uri, "smb"))) {
-			/* there is  a bug in the GIO sftp and smb module in glib version 2.18 that returns 'file exists error' 
+			/* there is  a bug in the GIO sftp and smb module in glib version 2.18 that returns 'file exists error'
 			   if you request an async content_replace */
 			GError *error2 = NULL;
 			g_file_replace_contents(cns->uri, cns->buffer->data, cns->buffer_size, cns->etag,
@@ -420,7 +418,7 @@ checkNsave_replace_async_lcb(GObject * source_object, GAsyncResult * res, gpoint
 			}
 		}
 #endif
-		DEBUG_MSG("checkNsave_replace_async_lcb, finished ");
+		DEBUG_MSG("checkNsave_replace_async_lcb, before callback, finished with ");
 		DEBUG_URI(cns->uri, TRUE);
 		cns->callback_func(CHECKANDSAVE_FINISHED, NULL, cns->callback_data);
 	}
@@ -490,10 +488,10 @@ file_checkNsave_uri_async(GFile * uri, GFileInfo * info, Trefcpointer * buffer,
 GFile *backup_uri_from_orig_uri(GFile * origuri) {
 	gchar *tmp, *tmp2;
 	GFile *ret;
-	
+
 	tmp = g_file_get_parse_name(origuri);
 	tmp2 = g_strconcat(tmp,"~",NULL);
-	
+
 	ret = g_file_parse_name(tmp2);
 	g_free(tmp);
 	g_free(tmp2);
@@ -505,6 +503,7 @@ GFile *backup_uri_from_orig_uri(GFile * origuri) {
 static void
 openfile_cleanup(Topenfile * of)
 {
+	DEBUG_MSG("openfile_cleanup, for of=%p\n",of);
 	g_object_unref(of->uri);
 	g_object_unref(of->cancel);
 	g_slice_free(Topenfile, of);
@@ -537,8 +536,9 @@ openfile_async_mount_lcb(GObject * source_object, GAsyncResult * res, gpointer u
 		/* open again */
 		g_file_load_contents_async(of->uri, of->cancel, openfile_async_lcb, of);
 	} else {
-		DEBUG_MSG("failed to mount with error %d %s!!\n", error->code, error->message);
+		DEBUG_MSG("failed to mount with error %d %s!!, call OPENFILE_ERROR for of=%p\n", error->code, error->message, of);
 		of->callback_func(OPENFILE_ERROR, error, NULL, 0, of->callback_data);
+		openfile_cleanup(of);
 	}
 	gmo = NULL;
 	resume_after_wait_for_mount();
@@ -625,7 +625,7 @@ fileintodoc_cleanup(Tfileintodoc * fid)
 }
 
 static gboolean
-fileintodoc_finished_idle_lcb(gpointer data) 
+fileintodoc_finished_idle_lcb(gpointer data)
 {
 	Tfileintodoc *fid = data;
 	DEBUG_MSG("fileintodoc_finished_idle_lcb, loading the data for doc %p\n",fid->doc);
@@ -665,7 +665,7 @@ fileintodoc_finished_idle_lcb(gpointer data)
 	refcpointer_unref(fid->buffer);
 	fileintodoc_cleanup(data);
 	return FALSE;
-} 
+}
 
 static void
 fileintodoc_lcb(Topenfile_status status, GError * gerror, Trefcpointer * buffer, goffset buflen, gpointer data)
@@ -673,7 +673,7 @@ fileintodoc_lcb(Topenfile_status status, GError * gerror, Trefcpointer * buffer,
 	Tfileintodoc *fid = data;
 	switch (status) {
 	case OPENFILE_FINISHED:
-		/* a GtkTextView with lots of data displays incredibly slow. So during 
+		/* a GtkTextView with lots of data displays incredibly slow. So during
 		startup of bluefish we want the GUI to show first, and *then* it should load
 		the data into the documents. That's why we insert this extra idle call with low
 		priority, to give the UI priority over the data loading */
@@ -775,7 +775,7 @@ file2doc_goto_idle_cb(Tfile2doc * f2d)
 	}
 	f2d->doc->goto_line = -1;
 	f2d->doc->goto_offset = -1;
-	f2d->doc->load = NULL;	
+	f2d->doc->load = NULL;
 	file2doc_cleanup(f2d);
 	return FALSE;
 }
@@ -791,7 +791,7 @@ file2doc_finished_idle_lcb(gpointer data)
 	DEBUG_MSG("file2doc_finished_idle_lcb started for doc %p\n",f2d->doc);
 	if (f2d->recovery_status == 1) {
 		GtkTextIter itstart, itend;
-	
+
 		f2d->recovery_status = 2;
 		doc_buffer_to_textbox(f2d->doc, f2d->buffer->data, f2d->buflen, FALSE, TRUE);
 		gtk_text_buffer_get_bounds(f2d->doc->buffer, &itstart, &itend);
@@ -896,6 +896,9 @@ file2doc_lcb(Topenfile_status status, GError * gerror, Trefcpointer * buffer, go
 static void
 fill_fileinfo_cleanup(Tfileinfo * fi)
 {
+	DEBUG_MSG("fill_fileinfo_cleanup for fi=%p\n", fi);
+	if (fi->doc && fi->doc->info)
+		fi->doc->info = NULL;
 	g_object_unref(fi->uri);
 	g_object_unref(fi->cancel);
 	g_slice_free(Tfileinfo, fi);
@@ -915,6 +918,7 @@ fill_fileinfo_async_mount_lcb(GObject * source_object, GAsyncResult * res, gpoin
 	if (g_file_mount_enclosing_volume_finish(fi->uri, res, &error)) {
 		fill_fileinfo_run(fi);
 	} else {
+		DEBUG_MSG("fill_fileinfo_async_mount_lcb, mount error for fi=%p\n", fi);
 		g_warning("failed to mount with error %d %s!!\n", error->code, error->message);
 		queue_worker_ready(&fiqueue);
 		fill_fileinfo_cleanup(fi);
@@ -938,9 +942,9 @@ fill_fileinfo_lcb(GObject * source_object, GAsyncResult * res, gpointer user_dat
 		}
 		fi->doc->fileinfo = info; /* no need to ref it, the call to g_file_query_info_finish returns a reference */
 #ifdef DEBUG
-		/* weird.. since I (Olivier) have a SSD in my laptop I sometimes receive wrong values 
+		/* weird.. since I (Olivier) have a SSD in my laptop I sometimes receive wrong values
 		   for the file size here. And then I get a 'file changed on disk' error.
-		   But the file did not change at all, just the first time the recorded size 
+		   But the file did not change at all, just the first time the recorded size
 		   was wrong. Right now I get 16384 bytes (2^14) for a 7918 byte file. */
 		if (g_file_info_has_attribute(info, G_FILE_ATTRIBUTE_STANDARD_SIZE)) {
 			gchar *curi = g_file_get_uri(fi->uri);
@@ -1082,7 +1086,7 @@ typedef struct {
 	gchar *extension_filter;
 	GPatternSpec *patspec;
 	GFile *topbasedir;			/* the top directory where advanced open started */
-	
+
 	void (*finished_cb) (gpointer data);
 	void (*filematch_cb) (const gchar *name, GFile *uri, gpointer data);
 	gpointer data;
@@ -1151,7 +1155,7 @@ enumerator_next_files_lcb(GObject * source_object, GAsyncResult * res, gpointer
 	Tfindfiles_dir *ffd = user_data;
 	GList *alldoclist;
 
-	if (ffd->ff->cancel) {	
+	if (ffd->ff->cancel) {
 		findfiles_load_directory_cleanup(ffd);
 		return;
 	}
@@ -1318,7 +1322,7 @@ typedef struct {
 	Tbfwin *bfwin;
 	gchar *content_filter;
 	gboolean use_regex;
-	GRegex *content_reg;	
+	GRegex *content_reg;
 } Topenadvanced;
 
 typedef struct {
@@ -1327,7 +1331,7 @@ typedef struct {
 	GFileInfo *finfo;
 } Topenadvanced_uri;
 
-static void 
+static void
 open_advanced_cleanup(Topenadvanced *oa) {
 	DEBUG_MSG("open_advanced_cleanup, oa=%p\n",oa);
 	if (oa->content_filter)
@@ -1357,7 +1361,7 @@ open_adv_open_uri_cleanup(Topenadvanced_uri * oau)
 	g_print("allocuri=%d\n", omemcount.allocuri);
 #endif							/* OAD_MEMCOUNT */
 	g_slice_free(Topenadvanced_uri, oau);
-	
+
 }
 
 static gboolean
@@ -1719,7 +1723,7 @@ static void
 progress_update(Tsync *sync, GFile *uri)
 {
 /*	if (g_timer_elapsed(sync->timer,NULL) > 0.05) */
-/*	if (sync->num_found % 10==0 || (sync->num_found - sync->num_finished) % 10 == 0) 
+/*	if (sync->num_found % 10==0 || (sync->num_found - sync->num_finished) % 10 == 0)
 	{*/
 	sync->progress_callback(uri, sync->num_found, sync->num_finished, sync->num_failed, sync->callback_data);
 /*		g_print("timer elapsed %f\n",g_timer_elapsed(sync->timer,NULL));
@@ -2093,7 +2097,7 @@ file_handle(GFile * uri, Tbfwin * bfwin, gchar * mimetype, gboolean external_inp
 				g_object_unref(docparent);
 			}
 			if (!curi) {
-				curi = g_file_get_uri(uri);
+				curi = g_file_get_basename(uri);
 			}
 			if (curi) {
 				tmp = g_strdup_printf("<img src=\"%s\" alt=\"\" />", curi);
