@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * file_dialogs.c - file dialogs
  *
- * Copyright (C) 2005-2011 Olivier Sessink
+ * Copyright (C) 2005-2012 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #include "bookmark.h"
 #include "dialog_utils.h"
 #include "document.h"
+#include "doc_text_tools.h"
 #include "file_autosave.h"
 #include "file.h"
 #include "filebrowser2.h"
@@ -81,7 +82,7 @@ files_advanced_win_ok_clicked(Tfiles_advanced * tfs)
 	content_filter = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(tfs->grep_pattern));
 	if (content_filter && content_filter[0]!='\0')
 		tfs->bfwin->session->searchlist = add_to_history_stringlist(tfs->bfwin->session->searchlist, content_filter, FALSE, TRUE);
-	if (extension_filter && extension_filter[0] != '\0') 
+	if (extension_filter && extension_filter[0] != '\0')
 		tfs->bfwin->session->filegloblist = add_to_history_stringlist(tfs->bfwin->session->filegloblist, extension_filter,FALSE, TRUE);
 
 	retval =
@@ -167,7 +168,7 @@ files_advanced_win(Tbfwin * bfwin, gchar * basedir)
 	dialog_mnemonic_label_in_table(_("Base _Dir:"), tfs->basedir, table, 0, 1, 0, 1);
 
 	button =
-		dialog_button_new_with_image_in_table(NULL, GTK_STOCK_OPEN, G_CALLBACK(files_advanced_win_select_basedir_lcb), tfs, 
+		dialog_button_new_with_image_in_table(NULL, GTK_STOCK_OPEN, G_CALLBACK(files_advanced_win_select_basedir_lcb), tfs,
 						TRUE, FALSE,
 						table, 5, 6, 0,1);
 
@@ -265,7 +266,7 @@ file_open_ok_lcb(GtkDialog * dialog, gint response, Tbfwin * bfwin)
 				DEBUG_MSG("file_open_ok_lcb, session encoding is set to %s\n", bfwin->session->encoding);
 			}
 		}
-#if GTK_CHECK_VERSION(2,14,0)
+
 		tmpslist = slist = gtk_file_chooser_get_files(GTK_FILE_CHOOSER(dialog));
 		while (tmpslist) {
 			doc_new_from_uri(bfwin, (GFile *) tmpslist->data, NULL, (slist->next != NULL), FALSE, -1, -1);
@@ -273,18 +274,6 @@ file_open_ok_lcb(GtkDialog * dialog, gint response, Tbfwin * bfwin)
 			tmpslist = tmpslist->next;
 		}
 		g_slist_free(slist);
-#else
-		tmpslist = slist = gtk_file_chooser_get_uris(GTK_FILE_CHOOSER(dialog));
-		while (tmpslist) {
-			GFile *file;
-			file = g_file_new_for_uri((gchar *) tmpslist->data);
-			doc_new_from_uri(bfwin, file, NULL, (slist->next != NULL), FALSE, -1, -1);
-			g_object_unref(file);
-			g_free((gchar *) tmpslist->data);
-			tmpslist = tmpslist->next;
-		}
-		g_slist_free(slist);
-#endif
 	}
 	gtk_widget_destroy(GTK_WIDGET(dialog));
 }
@@ -433,6 +422,7 @@ typedef struct {
 	Tdocument *doc;
 	GFile *unlink_uri;
 	GFile *fbrefresh_uri;
+	Tdocsave_mode savemode;
 } Tdocsavebackend;
 
 static void
@@ -459,7 +449,7 @@ doc_checkNsave_lcb(TcheckNsave_status status, GError * gerror, gpointer data)
 	Tdocsavebackend *dsb = data;
 	Tdocument *doc = dsb->doc;
 	gchar *errmessage;
-	DEBUG_MSG("doc_checkNsave_lcb, doc=%p, status=%d\n", doc, status);
+	DEBUG_MSG("doc_checkNsave_lcb, doc=%p, status=%d, doc->uri=%p\n", doc, status, doc->uri);
 	switch (status) {
 	case CHECKANDSAVE_ERROR_NOBACKUP:
 		if (main_v->props.backup_abort_action == 0) {
@@ -556,17 +546,21 @@ gtk_label_get_text(GTK_LABEL(doc->tab_label)));
 		} else {
 			/* YES! we're done! update the fileinfo ! */
 			gtk_text_view_set_editable(GTK_TEXT_VIEW(doc->view), TRUE);
-			DEBUG_MSG("doc_checkNsave_lcb, re-set async doc->fileinfo (current=%p)\n", doc->fileinfo);
-			if (doc->fileinfo)
-				g_object_unref(doc->fileinfo);
-			doc->fileinfo = NULL;
-			file_doc_fill_fileinfo(doc, doc->uri);
-			if (main_v->props.clear_undo_on_save) {
-				doc_unre_clear_all(doc);
+			if (dsb->savemode != docsave_copy) {
+				DEBUG_MSG("doc_checkNsave_lcb, re-set async doc->fileinfo (current=%p) for uri %p\n", doc->fileinfo, doc->uri);
+				if (doc->fileinfo)
+					g_object_unref(doc->fileinfo);
+				doc->fileinfo = NULL;
+				file_doc_fill_fileinfo(doc, doc->uri);
+				if (main_v->props.clear_undo_on_save) {
+					doc_unre_clear_all(doc);
+				} else {
+					doc_unre_clear_not_modified(doc);
+				}
+				doc_set_modified(doc, 0);
 			}
-			doc_set_modified(doc, 0);
 			/* in fact the filebrowser should also be refreshed if the document was closed, but
-			   when a document is closed, the filebrowser is anyway refreshed (hmm perhaps only if 
+			   when a document is closed, the filebrowser is anyway refreshed (hmm perhaps only if
 			   'follow document focus' is set). */
 			if (dsb->unlink_uri && dsb->fbrefresh_uri) {
 				GFile *parent1, *parent2;
@@ -608,21 +602,18 @@ gtk_label_get_text(GTK_LABEL(doc->tab_label)));
  * Return value: gchar* with newly allocated string, or NULL on failure or abort
  **/
 gchar *
-ask_new_filename(Tbfwin * bfwin, const gchar * old_curi, const gchar * gui_name, gboolean is_move)
+ask_new_filename(Tbfwin * bfwin, const gchar * old_curi, const gchar *dialogtext)
 {
 	Tdocument *exdoc;
 	GList *alldocs;
 	gchar *new_curi = NULL;
 	GFile *uri;
-	gchar *dialogtext;
 	GtkWidget *dialog;
 
-	dialogtext = g_strdup_printf((is_move) ? _("Move/rename %s to") : _("Save %s as"), gui_name);
+
 	dialog =
 		file_chooser_dialog(bfwin, dialogtext, GTK_FILE_CHOOSER_ACTION_SAVE, old_curi, FALSE, FALSE, NULL,
 							FALSE);
-	g_free(dialogtext);
-
 	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
 		new_curi = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(dialog));
 	}
@@ -680,17 +671,30 @@ ask_new_filename(Tbfwin * bfwin, const gchar * old_curi, const gchar * gui_name,
 	return new_curi;
 }
 
+static void
+session_set_savedir(Tbfwin * bfwin, GFile *uri)
+{
+	if (uri) {
+		GFile *parent = g_file_get_parent(uri);
+		if (bfwin->session->savedir)
+			g_free(bfwin->session->savedir);
+		bfwin->session->savedir = g_file_get_uri(parent);
+	}
+}
+
 void
-doc_save_backend(Tdocument * doc, gboolean do_save_as, gboolean do_move, gboolean close_doc,
+doc_save_backend(Tdocument * doc, Tdocsave_mode savemode, gboolean close_doc,
 				 gboolean close_window)
 {
 	gchar *tmp;
 	Trefcpointer *buffer;
-	gchar *curi = NULL;
 	Tdocsavebackend *dsb;
+	GFile *dest_uri=NULL;
+	GFileInfo *dest_finfo=NULL;
+	gboolean firstsave;
 	DEBUG_MSG
-		("doc_save_backend, started for doc %p, save_as=%d, do_move=%d, close_doc=%d, close_window=%d\n", doc,
-		 do_save_as, do_move, close_doc, close_window);
+		("doc_save_backend, started for doc %p, mode=%d, close_doc=%d, close_window=%d\n", doc,
+		 savemode, close_doc, close_window);
 
 	if (doc->readonly) {
 		g_print("Cannot save readonly document !?!?");
@@ -699,7 +703,7 @@ doc_save_backend(Tdocument * doc, gboolean do_save_as, gboolean do_move, gboolea
 
 	dsb = g_new0(Tdocsavebackend, 1);
 	dsb->doc = doc;
-
+	dsb->savemode = savemode;
 	/* should be moved to a plugin interface, because this is HTML specific */
 	/* update author meta tag */
 	if (main_v->props.auto_update_meta_author) {
@@ -709,7 +713,7 @@ doc_save_backend(Tdocument * doc, gboolean do_save_as, gboolean do_move, gboolea
 			author_tmp = g_strconcat("<meta name=\"author\" content=\"", realname, "\" ", NULL);
 			snr3_run_extern_replace(doc,
 									"<meta[ \t\n]+name[ \t\n]*=[ \t\n]*\"author\"[ \t\n]+content[ \t\n]*=[ \t\n]*\"[^\"]*\"[ \t\n]*",
-									snr3scope_doc, snr3type_pcre, FALSE, author_tmp, FALSE);
+									snr3scope_doc, snr3type_pcre, FALSE, author_tmp, FALSE, FALSE);
 			g_free(author_tmp);
 		}
 	}
@@ -743,7 +747,7 @@ doc_save_backend(Tdocument * doc, gboolean do_save_as, gboolean do_move, gboolea
 		date_tmp = g_strconcat("<meta name=\"date\" content=\"", isotime, "\" ", NULL);
 		snr3_run_extern_replace(doc,
 								"<meta[ \t\n]+name[ \t\n]*=[ \t\n]*\"date\"[ \t\n]+content[ \t\n]*=[ \t\n]*\"[^\"]*\"[ \t\n]*",
-								snr3scope_doc, snr3type_pcre, FALSE, date_tmp, FALSE);
+								snr3scope_doc, snr3type_pcre, FALSE, date_tmp, FALSE, FALSE);
 		g_free(date_tmp);
 	}
 
@@ -752,11 +756,11 @@ doc_save_backend(Tdocument * doc, gboolean do_save_as, gboolean do_move, gboolea
 		snr3_run_extern_replace(doc,
 								"<meta[ \t\n]+name[ \t\n]*=[ \t\n]*\"generator\"[ \t\n]+content[ \t\n]*=[ \t\n]*\"[^\"]*\"[ \t\n]*",
 								snr3scope_doc, snr3type_pcre, FALSE,
-								"<meta name=\"generator\" content=\"Bluefish " VERSION "\" ", FALSE);
+								"<meta name=\"generator\" content=\"Bluefish " VERSION "\" ", FALSE, FALSE);
+	}
+	if (main_v->props.strip_trailing_spaces_on_save) {
+		strip_trailing_spaces(doc);
 	}
-
-	if (doc->uri)
-		curi = g_file_get_uri(doc->uri);
 
 	if (doc->save) {
 		gchar *errmessage;
@@ -767,48 +771,69 @@ doc_save_backend(Tdocument * doc, gboolean do_save_as, gboolean do_move, gboolea
 		message_dialog_new(BFWIN(doc->bfwin)->main_window, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
 						   _("Save in progress!"), errmessage);
 		g_free(errmessage);
+		g_free(dsb);
+		DEBUG_MSG("doc_save_backend, already save in progress, return\n");
 		return;
 	}
-	if (doc->uri == NULL) {
-		do_save_as = 1;
-	}
-	if (do_move) {
-		do_save_as = 1;
-	}
-	if (do_save_as) {
-		gchar *newfilename;
+	firstsave = (doc->uri == NULL && savemode == docsave_normal);
+	if (firstsave || savemode != docsave_normal) {
+		gchar *newfilename, *curi, *dialogtext;
+		const gchar *gui_name = gtk_label_get_text(GTK_LABEL(doc->tab_label));
+		DEBUG_MSG("doc_save_backend, no uri (doc->uri=%p), or saveas/copy/move (savemode=%d)\n", doc->uri, savemode);
+		curi = doc->uri ? g_file_get_uri(doc->uri) : NULL;
+		if (savemode == docsave_normal) {
+			dialogtext = g_strdup(_("Save"));
+		} else if (savemode == docsave_saveas){
+			dialogtext = g_strdup_printf(_("Save %s as"), gui_name);
+		} else if (savemode == docsave_move){
+			dialogtext = g_strdup_printf(_("Move/rename %s to"), gui_name);
+		} else {
+			dialogtext = g_strdup_printf(_("Save copy of %s"), gui_name);
+		}
 		newfilename =
-			ask_new_filename(BFWIN(doc->bfwin), curi, gtk_label_get_text(GTK_LABEL(doc->tab_label)), do_move);
+			ask_new_filename(BFWIN(doc->bfwin), curi, dialogtext);
+		g_free(dialogtext);
+		g_free(curi);
 		if (!newfilename) {
-			if (curi)
-				g_free(curi);
+			DEBUG_MSG("doc_save_backend, no newfilename, return\n");
 			g_free(dsb);
 			return;
 		}
-		if (doc->uri) {
-			if (do_move) {
-				dsb->unlink_uri = doc->uri;	/* unlink this uri later */
-				g_object_ref(dsb->unlink_uri);
+		dest_uri = g_file_new_for_uri(newfilename);
+		if (doc->uri == NULL || savemode == docsave_saveas || savemode == docsave_move) {
+			if (doc->uri) {
+				if (savemode == docsave_move) {
+					dsb->unlink_uri = doc->uri;	/* unlink and refresh this uri later */
+					g_object_ref(dsb->unlink_uri);
+				}
+				g_object_unref(doc->uri);
 			}
-			g_object_unref(doc->uri);
+			doc->uri = dest_uri;
+			g_object_ref(doc->uri);
+			dest_finfo = doc->fileinfo;
 		}
-
-		doc->uri = g_file_new_for_uri(newfilename);
-		if (do_move)
-			bmark_doc_renamed(BFWIN(doc->bfwin), doc);
-
-		if (curi)
-			g_free(curi);
-		curi = newfilename;
-		DEBUG_MSG("doc_save_backend, new uri=%s\n", curi);
-		dsb->fbrefresh_uri = doc->uri;	/* refresh this uri later */
-		g_object_ref(dsb->fbrefresh_uri);
+		DEBUG_MSG("doc_save_backend, newfilename=%s, dest_uri=%p, doc->uri=%p\n",newfilename, dest_uri, doc->uri);
+		g_free(newfilename);
+	} else {
+		DEBUG_MSG("doc_save_backend, have uri and normal save\n");
+		dest_uri = doc->uri;
+		dest_finfo = doc->fileinfo;
+		g_object_ref(dest_uri);
 	}
-	session_set_savedir(doc->bfwin, curi);
+	DEBUG_MSG("doc_save_backend, dest_uri=%p\n",dest_uri);
+	if (doc->uri && (savemode == docsave_move || savemode==docsave_saveas)) {
+		bmark_doc_renamed(BFWIN(doc->bfwin), doc);
+	}
+	if ((firstsave || savemode != docsave_normal)&& dest_uri) {
+		dsb->fbrefresh_uri = dest_uri;	/* refresh this uri later */
+		g_object_ref(dest_uri);
+	}
+
+	session_set_savedir(doc->bfwin, dest_uri);
 
 	tmp = doc_get_buffer_in_encoding(doc);
 	if (!tmp) {
-		g_free(curi);
+		g_free(dsb);
 		return;
 	}
 #if !GLIB_CHECK_VERSION(2, 18, 0)
@@ -826,20 +851,19 @@ glib_major_version, glib_minor_version, glib_micro_version);
 	doc->close_doc = close_doc;
 	doc->close_window = close_window;
 	gtk_text_view_set_editable(GTK_TEXT_VIEW(doc->view), FALSE);
-	DEBUG_MSG("doc_save_backend, calling file_checkNsave_uri_async for %zd bytes\n", strlen(buffer->data));
+	DEBUG_MSG("doc_save_backend, calling file_checkNsave_uri_async with uri %p for %zd bytes\n", dest_uri, strlen(buffer->data));
 	doc->save =
-		file_checkNsave_uri_async(doc->uri, doc->fileinfo, buffer, strlen(buffer->data), !do_save_as,
+		file_checkNsave_uri_async(dest_uri, dest_finfo, buffer, strlen(buffer->data), savemode == docsave_normal,
 								  main_v->props.backup_file, doc_checkNsave_lcb, dsb);
 
-	if (do_save_as) {
+	if (firstsave || savemode == docsave_saveas || savemode == docsave_move) {
 		doc->readonly = FALSE;
 		doc_reset_filetype(doc, doc->uri, buffer->data, strlen(buffer->data));
 		doc_set_title(doc);
 		doc_force_activate(doc);
 	}
+	g_object_unref(dest_uri);
 	refcpointer_unref(buffer);
-
-	g_free(curi);
 }
 
 /**
@@ -855,7 +879,7 @@ void
 file_save_cb(GtkWidget * widget, Tbfwin * bfwin)
 {
 	if (bfwin->current_document)
-		doc_save_backend(bfwin->current_document, FALSE, FALSE, FALSE, FALSE);
+		doc_save_backend(bfwin->current_document, docsave_normal, FALSE, FALSE);
 }
 
 /**
@@ -871,7 +895,7 @@ void
 file_save_as_cb(GtkWidget * widget, Tbfwin * bfwin)
 {
 	if (bfwin->current_document)
-		doc_save_backend(bfwin->current_document, TRUE, FALSE, FALSE, FALSE);
+		doc_save_backend(bfwin->current_document, docsave_saveas, FALSE, FALSE);
 }
 
 /**
@@ -887,7 +911,7 @@ void
 file_move_to_cb(GtkWidget * widget, Tbfwin * bfwin)
 {
 	if (bfwin->current_document)
-		doc_save_backend(bfwin->current_document, TRUE, TRUE, FALSE, FALSE);
+		doc_save_backend(bfwin->current_document, docsave_move, FALSE, FALSE);
 }
 
 void
@@ -900,7 +924,7 @@ file_save_all(Tbfwin * bfwin)
 	while (tmplist) {
 		tmpdoc = (Tdocument *) tmplist->data;
 		if (tmpdoc->modified) {
-			doc_save_backend(tmpdoc, FALSE, FALSE, FALSE, FALSE);
+			doc_save_backend(tmpdoc, docsave_normal, FALSE, FALSE);
 		}
 		tmplist = g_list_next(tmplist);
 	}
@@ -925,7 +949,7 @@ file_save_all_cb(GtkWidget * widget, Tbfwin * bfwin)
 	while (tmplist) {
 		tmpdoc = (Tdocument *) tmplist->data;
 		if (tmpdoc->modified) {
-			doc_save_backend(tmpdoc, FALSE, FALSE, FALSE, FALSE);
+			doc_save_backend(tmpdoc, docsave_normal, FALSE, FALSE);
 		}
 		tmplist = g_list_next(tmplist);
 	}
@@ -937,7 +961,7 @@ doc_save_all_close(Tbfwin * bfwin)
 	GList *tmplist = g_list_first(bfwin->documentlist);
 	while (tmplist) {
 		Tdocument *tmpdoc = (Tdocument *) tmplist->data;
-		doc_save_backend(tmpdoc, FALSE, FALSE, TRUE, TRUE);
+		doc_save_backend(tmpdoc, docsave_normal, TRUE, TRUE);
 		tmplist = g_list_next(tmplist);
 	}
 }
@@ -968,7 +992,7 @@ multiple_files_modified_dialog(Tbfwin * bfwin)
 	return (Tclose_mode) retval;
 }
 
-/* return TRUE if all are either closed or saved 
+/* return TRUE if all are either closed or saved
 return FALSE on cancel*/
 gboolean
 choose_per_file(Tbfwin * bfwin, gboolean close_window)
@@ -993,7 +1017,7 @@ choose_per_file(Tbfwin * bfwin, gboolean close_window)
 				break;
 			case 2:			/* save */
 				DEBUG_MSG("choose_per_file, call doc_save\n");
-				doc_save_backend(tmpdoc, FALSE, FALSE, TRUE, close_window);
+				doc_save_backend(tmpdoc, docsave_normal, TRUE, close_window);
 				break;
 			}
 		} else {
@@ -1015,7 +1039,7 @@ doc_close_single_backend(Tdocument * doc, gboolean delay_activate, gboolean clos
 		remove_autosave(doc);
 	if (doc->load != NULL || doc->info != NULL) {
 		/* we should cancel the action now..., and then let the callbacks close it...
-		   the order is important, because the info callback will not close the document, 
+		   the order is important, because the info callback will not close the document,
 		   only the load callback will call doc_close_single_backend */
 		doc->close_doc = TRUE;
 		doc->close_window = close_window;
@@ -1023,7 +1047,7 @@ doc_close_single_backend(Tdocument * doc, gboolean delay_activate, gboolean clos
 			file_asyncfileinfo_cancel(doc->info);
 		if (doc->load)
 			file2doc_cancel(doc->load);
-		/* we will not cancel save operations, because it might corrupt the file, let 
+		/* we will not cancel save operations, because it might corrupt the file, let
 		   them just timeout */
 		DEBUG_MSG("doc_close_single_backend, cancelled load/info and set close_doc to TRUE, returning now\n");
 		return FALSE;
@@ -1048,7 +1072,7 @@ doc_close_single_backend(Tdocument * doc, gboolean delay_activate, gboolean clos
 			return FALSE;
 			break;
 		case 2:
-			doc_save_backend(doc, FALSE, FALSE, TRUE, close_window);
+			doc_save_backend(doc, docsave_normal, TRUE, close_window);
 			break;
 		}
 	} else {
@@ -1093,7 +1117,7 @@ doc_close_multiple_backend(Tbfwin * bfwin, gboolean close_window, Tclose_mode cl
 		}
 	}*/
 
-	/* we duplicate the documentlist so we can safely walk trough the list, in 
+	/* we duplicate the documentlist so we can safely walk trough the list, in
 	   our duplicate list there is no chance that the list is changed during the time
 	   we walk the list */
 	duplist = g_list_copy(bfwin->documentlist);
@@ -1105,7 +1129,7 @@ doc_close_multiple_backend(Tbfwin * bfwin, gboolean close_window, Tclose_mode cl
 			tmpdoc->modified = FALSE;
 			doc_close_single_backend(tmpdoc, TRUE, close_window);
 		} else if (close_mode == close_mode_save_all) {
-			doc_save_backend(tmpdoc, FALSE, FALSE, TRUE, close_window);
+			doc_save_backend(tmpdoc, docsave_normal, TRUE, close_window);
 		}
 		tmplist = g_list_next(tmplist);
 	}
@@ -1140,7 +1164,7 @@ file_close_all(Tbfwin * bfwin)
 /**
  * file_close_all_cb:
  * @widget: unused #GtkWidget
- * @bfwin: #Tbfwin* 
+ * @bfwin: #Tbfwin*
  *
  * Close all open files. Prompt user when neccessary.
  *
@@ -1361,14 +1385,14 @@ sync_dialog(Tbfwin * bfwin)
 	gtk_table_attach(GTK_TABLE(table), file_but_new2(sd->entry_remote, 1, bfwin, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER),
 									2, 3, 1, 2, GTK_FILL, GTK_FILL, 3, 3);
 
-	sd->delete_deprecated = dialog_check_button_in_table(_("Delete deprecated files"), 
+	sd->delete_deprecated = dialog_check_button_in_table(_("Delete deprecated files"),
 						bfwin->session->sync_delete_deprecated, table,
 						0, 3, 2, 3);
-	
-	sd->include_hidden = dialog_check_button_in_table(_("Include hidden files"), 
+
+	sd->include_hidden = dialog_check_button_in_table(_("Include hidden files"),
 						bfwin->session->sync_include_hidden, table,
 						0, 3, 3, 4);
-	
+
 	sd->messagelabel = gtk_label_new(NULL);
 	gtk_box_pack_start(GTK_BOX(carea), sd->messagelabel, FALSE, FALSE, 4);
 
@@ -1397,7 +1421,8 @@ modified_on_disk_warning_string(const gchar * filename, GFileInfo * oldfinfo, GF
 {
 	gchar *tmpstr, *oldtimestr, *newtimestr;
 	time_t newtime, oldtime;
-	gsize oldsize, newsize;
+	goffset oldsize, newsize;
+	gchar *strnewsize, *stroldsize;
 
 	newtime = (time_t) g_file_info_get_attribute_uint64(newfinfo, G_FILE_ATTRIBUTE_TIME_MODIFIED);
 	oldtime = (time_t) g_file_info_get_attribute_uint64(oldfinfo, G_FILE_ATTRIBUTE_TIME_MODIFIED);
@@ -1405,14 +1430,18 @@ modified_on_disk_warning_string(const gchar * filename, GFileInfo * oldfinfo, GF
 	oldtimestr = bf_portable_time(&oldtime);
 	newsize = g_file_info_get_size(newfinfo);
 	oldsize = g_file_info_get_size(oldfinfo);
+	strnewsize = g_strdup_printf("%"G_GOFFSET_FORMAT"", newsize);
+	stroldsize = g_strdup_printf("%"G_GOFFSET_FORMAT"", oldsize);
 	/*g_print("oldtimestr=%s, newtimestr=%s\n",oldtimestr,newtimestr); */
 	tmpstr = g_strdup_printf(_("Filename:%s changed on disk.\n\n"
 							   "Original modification time was %s\n"
 							   "New modification time is %s\n"
-							   "Original size was %lu\n"
-							   "New size is %lu"), filename, oldtimestr, newtimestr, (long unsigned int)oldsize, (long unsigned int)newsize);
+							   "Original size was %s bytes\n"
+							   "New size is %s bytes"), filename, oldtimestr, newtimestr, stroldsize, strnewsize);
 	g_free(newtimestr);
 	g_free(oldtimestr);
+	g_free(strnewsize);
+	g_free(stroldsize);
 	return tmpstr;
 }
 
@@ -1435,7 +1464,7 @@ doc_activate_modified_lcb(Tcheckmodified_status status, GError * gerror, GFileIn
 											  buttons, _("File disappeared from disk\n"), tmpstr);
 			g_free(tmpstr);
 			if (retval == 1) {	/* save */
-				doc_save_backend(doc, FALSE, FALSE, FALSE, FALSE);
+				doc_save_backend(doc, docsave_normal, FALSE, FALSE);
 			} else {			/* unset */
 				document_unset_filename(doc);
 			}
