@@ -109,7 +109,7 @@ html_diag_new(Tbfwin * bfwin, gchar * title)
 
 	dg->range.pos = -1;
 	dg->range.end = -1;
-	if (main_v->props.transient_htdialogs) {
+	if (htmlbar_v.transient_htdialogs) {
 		/* must be set before realizing */
 		DEBUG_MSG("html_diag_new, setting dg->dialog=%p transient!\n", dg->dialog);
 		gtk_window_set_transient_for(GTK_WINDOW(dg->dialog), GTK_WINDOW(bfwin->main_window));
@@ -134,12 +134,18 @@ html_diag_table_in_vbox(Thtml_diag * dg, gint rows, gint cols)
 void
 html_diag_finish(Thtml_diag * dg, GCallback ok_func)
 {
-	GtkWidget *hbox;
+	GtkWidget *align, *hbox;
 
-	gtk_box_pack_start(GTK_BOX(dg->vbox), gtk_hseparator_new(), FALSE, FALSE, 12);
+	align = gtk_alignment_new(0.5, 1.0, 1.0, 0.0);
+	gtk_alignment_set_padding(GTK_ALIGNMENT(align), 12, 0, 0, 0);
+#if GTK_CHECK_VERSION(3,0,0)
+	hbox = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
+#else
 	hbox = gtk_hbutton_box_new();
+#endif
 	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
 	gtk_box_set_spacing(GTK_BOX(hbox), 12);
+	gtk_container_add(GTK_CONTAINER(align), hbox);
 
 	dg->obut = bf_stock_ok_button(ok_func, dg);
 	dg->cbut = bf_stock_cancel_button(G_CALLBACK(html_diag_cancel_clicked_cb), dg);
@@ -148,7 +154,7 @@ html_diag_finish(Thtml_diag * dg, GCallback ok_func)
 	gtk_box_pack_start(GTK_BOX(hbox), dg->obut, FALSE, FALSE, 0);
 	gtk_window_set_default(GTK_WINDOW(dg->dialog), dg->obut);
 
-	gtk_box_pack_start(GTK_BOX(dg->vbox), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(dg->vbox), align, FALSE, FALSE, 0);
 	gtk_widget_show_all(GTK_WIDGET(dg->dialog));
 }
 
@@ -156,6 +162,18 @@ html_diag_finish(Thtml_diag * dg, GCallback ok_func)
 /********** HTML -> DIALOG FUNCTIONS *************/
 /*************************************************/
 
+GtkWidget *
+html_diag_combobox_with_popdown(const gchar *setstring, GList *poplist, gboolean editable)
+{
+	return combobox_with_popdown(setstring?setstring:"", poplist, editable);
+}
+
+GtkWidget *
+html_diag_combobox_with_popdown_sized(const gchar *setstring, GList *poplist, gboolean editable, gint width)
+{
+	return combobox_with_popdown_sized(setstring?setstring:"", poplist, editable, width);
+}
+
 void
 parse_html_for_dialogvalues(gchar * dialogitems[], gchar * dialogvalues[]
 							, gchar ** custom, Ttagpopup * data)
@@ -288,6 +306,7 @@ parse_integer_for_dialog(gchar * valuestring, GtkWidget * spin, GtkWidget * entr
 		value = strtod(valuestring, NULL);
 	}
 	if (spin) {
+		gtk_entry_set_text(GTK_ENTRY (spin), "0");
 		gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), value);
 	}
 
@@ -447,7 +466,7 @@ generic_class_id_style_section(Thtml_diag * dg, gint firstattrwidget, GtkWidget
 	GtkWidget *but;
 
 	dg->attrwidget[firstattrwidget] =
-		combobox_with_popdown(tagvalues[firsttagvalue], dg->bfwin->session->classlist, 1);
+		html_diag_combobox_with_popdown(tagvalues[firsttagvalue], dg->bfwin->session->classlist, 1);
 	dialog_mnemonic_label_in_table(_("Cl_ass:"), dg->attrwidget[firstattrwidget], dgtable, 0, 1,
 								   firstrowintable + 0, firstrowintable + 1);
 	gtk_table_attach(GTK_TABLE(dgtable), dg->attrwidget[firstattrwidget], 1, 3, firstrowintable + 0,
