@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * project.c - project functionality
  *
- * Copyright (C) 2003-2011 Olivier Sessink
+ * Copyright (C) 2003-2012 Olivier Sessink
  * Copyright (C) 2011 James Hayward
  *
  * This program is free software; you can redistribute it and/or modify
@@ -65,6 +65,7 @@ free_session(Tsessionvars * session)
 	free_arraylist(session->bmarks);
 	free_stringlist(session->recent_files);
 	free_stringlist(session->recent_dirs);
+	g_free(session->ssearch_text);
 	g_free(session->opendir);
 	g_free(session->savedir);
 	g_free(session->encoding);
@@ -105,7 +106,7 @@ project_setup_initial_session(Tsessionvars * session, gboolean before_parse)
 
 	/* THE NUBER OF BYTES THAT IS COPIED HERE MUST BE EQUAL TO THE NUMBER OF INTEGERS
 	   IN THE STRUCT IN bluefish.h ELSE ALL WILL FAIL */
-	memcpy(session, main_v->session, 39 * sizeof(gint));
+	memcpy(session, main_v->session, 44 * sizeof(gint));
 }
 
 Tbfwin *
@@ -267,9 +268,9 @@ project_save(Tbfwin * bfwin, gboolean save_as)
 			  g_list_length(bfwin->project->files));
 	update_project_filelist(bfwin, bfwin->project);
 
-	bfwin->project->session->searchlist = limit_stringlist(bfwin->project->session->searchlist, 10, TRUE);
+	/*bfwin->project->session->searchlist = limit_stringlist(bfwin->project->session->searchlist, 10, TRUE);
 	bfwin->project->session->replacelist = limit_stringlist(bfwin->project->session->replacelist, 10, TRUE);
-	bfwin->project->session->urllist = limit_stringlist(bfwin->project->session->urllist, 10, TRUE);
+	bfwin->project->session->urllist = limit_stringlist(bfwin->project->session->urllist, 10, TRUE);*/
 	bmark_store_all(bfwin);
 
 	if (save_as || bfwin->project->uri == NULL) {
@@ -278,7 +279,7 @@ project_save(Tbfwin * bfwin, gboolean save_as)
 		GFile *newuri = NULL;
 		dialog =
 			file_chooser_dialog(bfwin, _("Enter Bluefish project filename"), GTK_FILE_CHOOSER_ACTION_SAVE,
-								NULL, TRUE, FALSE, "bfproject", FALSE);
+								bfwin->project->name, TRUE, FALSE, "bfproject", FALSE);
 		if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
 			gchar *filename;
 			newuri = gtk_file_chooser_get_file(GTK_FILE_CHOOSER(dialog));
