@@ -409,7 +409,7 @@ props_init_main(GHashTable * config_rc)
 	init_prop_string(&config_rc, &main_v->props.editor_font_string, "editor_font_string:", "monospace 10");
 	init_prop_integer(&config_rc, &main_v->props.editor_smart_cursor, "editor_smart_cursor:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.editor_tab_indent_sel, "editor_tab_indent_sel:", 0, TRUE);
-	init_prop_integer(&config_rc, &main_v->props.editor_auto_close_brackets, "editor_auto_close_brackets:", 1, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.editor_auto_close_brackets, "editor_auto_close_brackets:", 2, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.use_system_tab_font, "use_system_tab_font:", 1, TRUE);
 	init_prop_string(&config_rc, &main_v->props.tab_font_string, "tab_font_string:", "");
 	init_prop_string(&config_rc, &main_v->props.tab_color_modified, "tab_color_modified:", "#0000FF");
@@ -455,6 +455,8 @@ props_init_main(GHashTable * config_rc)
 	init_prop_integer(&config_rc, &main_v->props.auto_update_meta_date, "auto_update_meta_date:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.auto_update_meta_generator, "auto_update_meta_generator:", 1,
 					  TRUE);
+	init_prop_integer(&config_rc, &main_v->props.strip_trailing_spaces_on_save, "strip_trailing_spaces_on_save:", 0,
+					  TRUE);
 	init_prop_integer(&config_rc, &main_v->props.encoding_search_Nbytes, "encoding_search_Nbytes:", 500,
 					  TRUE);
 	init_prop_integer(&config_rc, &main_v->props.max_window_title, "max_window_title:", 0,TRUE);
@@ -509,7 +511,10 @@ props_init_main(GHashTable * config_rc)
 	init_prop_string(&config_rc, &main_v->props.autosave_file_prefix, "autosave_file_prefix:", "");
 	init_prop_string(&config_rc, &main_v->props.autosave_file_suffix, "autosave_file_suffix:", "#");
 	init_prop_string(&config_rc, &main_v->props.language, "language:", "");
-	init_prop_arraylist(&config_rc, &main_v->props.templates, "templates:", 2, TRUE);
+
+	init_prop_integer(&config_rc, &main_v->props.adv_textview_right_margin, "adv_textview_right_margin:", 1, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.adv_textview_left_margin, "adv_textview_left_margin:", 1, TRUE);
+	/*init_prop_arraylist(&config_rc, &main_v->props.templates, "templates:", 2, TRUE);*/
 	return config_rc;
 }
 
@@ -572,7 +577,7 @@ migrate_config_files(GHashTable * main_configlist, GFile * newrc)
 	oldsession = user_bfdir(OLDSESSION);
 	newsession = user_bfdir(CURSESSION);
 	merge_config_files(oldrc, oldsession, newrc, newsession);
-	/* add some entries to main_configlist. because main_configlist is used for 
+	/* add some entries to main_configlist. because main_configlist is used for
 	   saving as well this means we'll save these entries too */
 	if (parse_config_file(main_configlist, newrc)) {
 		/* are there any entries that we want to convert ?? */
@@ -605,7 +610,7 @@ convert_old_placeholders(gchar *orig, gchar old1, gchar new1, gchar old2, gchar
 
 /* on array length = newlen-1 we update to the new version
 
-if (overwrite) we free the list and replace it with the default values 
+if (overwrite) we free the list and replace it with the default values
 
 if (!overwrite) we overwrite all bluefish_defined options, and we append any new options
 */
@@ -613,10 +618,10 @@ static GList *update_externals(GList *current, GList *defaults, gboolean overwri
 {
 	GList *tmplist;
 	GHashTable *ht=NULL;
-	
+
 	if (current == NULL)
 		overwrite = TRUE;
-	
+
 	if (overwrite) {
 		free_arraylist(current);
 		current = NULL;
@@ -628,16 +633,16 @@ static GList *update_externals(GList *current, GList *defaults, gboolean overwri
 			gint len;
 			GList *cur;
 			len = g_strv_length(arr);
-	
+
 			cur = tmplist;
 			tmplist = g_list_next(tmplist);
-			
+
 			if (len == (newlen-1)) { /* convert the old (2.2.2 or older) format into the new format */
 				gchar **oldarr = arr;
 				/*g_print("prepend %s in front of %s\n",USER_DEFINED_ENABLED,arr[0]);*/
 				cur->data = arr = prepend_array(USER_DEFINED_ENABLED,arr);
 				g_strfreev(oldarr);
-				/* arr[2] contains the commandstring. now replace %I and %O (previously fifo in and 
+				/* arr[2] contains the commandstring. now replace %I and %O (previously fifo in and
 				fifo out) to temporary file in %i and out %o */
 				arr[commandindex] = convert_old_placeholders(arr[commandindex], 'O', 'o', 'I', 'i');
 				len = newlen;
@@ -675,7 +680,6 @@ GList * update_outputbox(GList *current, gboolean overwrite) {
 						  array_from_arglist(_("php"),
 											 "(.*) in (.*:\\\\[a-zA-Z0-9\\\\_.-]+) on line ([0-9]+)", "2",
 											 "3", "1", "php '%f'|", NULL));
-	}
 #else
 	defaults =
 		g_list_append(defaults,
@@ -715,6 +719,10 @@ GList * update_outputbox(GList *current, gboolean overwrite) {
 		g_list_append(defaults,
 						array_from_arglist(_("php codesniffer"), ":([0-9:]+):(.*)", "-1",
 										 "1", "2", "phpcs --report=emacs '%f'|", NULL));
+	defaults =
+		g_list_append(defaults,
+						array_from_arglist(_("pylint"), "(C|R|E|F|W):[ ]*([0-9]+),[0-9]+:(.*)", "-1",
+										 "2", "0", "pylint '%f'|", NULL));
 #endif
 	retlist = update_externals(current, defaults, overwrite, 7, 6);
 	free_arraylist(defaults);
@@ -737,7 +745,9 @@ GList *update_filters(GList *current, gboolean overwrite)
 		g_list_append(defaults,
 					  array_from_arglist(_("Dos2unix"), "|dos2unix|", NULL));
 	defaults =
-		g_list_append(defaults, array_from_arglist(_("Tidy HTML"), "|tidy|", NULL));
+		g_list_append(defaults, array_from_arglist(_("Tidy HTML"), "|tidy -utf8|", NULL));
+	defaults =
+		g_list_append(defaults, array_from_arglist(_("Tidy convert to XHTML"), "|tidy -utf8 -i -w 80 -c -q -asxhtml|", NULL));
 	defaults =
 		g_list_append(defaults,
 					  array_from_arglist(_("Tidy HTML (perltidy)"), "|perltidy -b|", NULL));
@@ -747,7 +757,27 @@ GList *update_filters(GList *current, gboolean overwrite)
 	defaults =
 		g_list_append(defaults,
 					  array_from_arglist(_("Render HTML to text"), "lynx -force_html -dump %i |", NULL));
-
+	defaults =
+		g_list_append(defaults,
+					  array_from_arglist(_("PHP Beautifier"), "|php_beautifier -t|", NULL));
+#ifdef WIN32
+	defaults =
+		g_list_append(defaults,
+					  array_from_arglist(_("Javascript Minify with jsmin"), "|python "PKGDATADIR"/jsmin.py|", NULL));
+	defaults =
+		g_list_append(defaults,
+					  array_from_arglist(_("CSS Minify with cssmin"), "|python "PKGDATADIR"/cssmin.py|", NULL));
+#else
+	defaults =
+		g_list_append(defaults,
+					  array_from_arglist(_("Javascript Minify with jsmin"), "|"PKGDATADIR"/jsmin.py|", NULL));
+	defaults =
+		g_list_append(defaults,
+					  array_from_arglist(_("CSS Minify with cssmin"), "|"PKGDATADIR"/cssmin.py|", NULL));
+#endif
+	defaults =
+		g_list_append(defaults,
+					  array_from_arglist(_("CSS tidy"), "|csstidy --preserve_css=true -|", NULL));
 	retlist = update_externals(current, defaults, overwrite, 3, 2);
 	free_arraylist(defaults);
 	return retlist;
@@ -756,7 +786,7 @@ GList *update_filters(GList *current, gboolean overwrite)
 GList *update_commands(GList *current, gboolean overwrite)
 {
 	GList *defaults=NULL, *retlist;
-#ifdef WIN32	
+#ifdef WIN32
 	defaults =
 		g_list_prepend(defaults,
 					  array_from_arglist(_("Windows XP Firefox"),
@@ -853,7 +883,7 @@ rcfile_parse_main(void)
 	file = user_bfdir(CURCONFIG);
 	if (!parse_config_file(main_configlist, file)) {
 		g_warning("no configfile %s, try to convert config files from older versions\n", CURCONFIG);
-		/* probably there is no configfile. try to migrate the configfile from a previous 
+		/* probably there is no configfile. try to migrate the configfile from a previous
 		   version */
 		migrate_config_files(main_configlist, file);
 	}
@@ -862,12 +892,13 @@ rcfile_parse_main(void)
 		main_v->props.encoding_search_Nbytes = 2048;
 
 	/* do some default configuration for the lists */
-	if (!main_v->props.config_version || strlen(main_v->props.config_version)<5 ||main_v->props.config_version[0] < '2' || main_v->props.config_version[2] < '2' || main_v->props.config_version[4] < '3') {
+	if (!main_v->props.config_version || strlen(main_v->props.config_version)<5 ||main_v->props.config_version[0] < '2' || main_v->props.config_version[2] < '2' || main_v->props.config_version[4] < '4') {
+		main_v->props.rcfile_from_old_version = 1;
 		main_v->props.external_command = update_commands(main_v->props.external_command, FALSE);
 		main_v->props.external_filter = update_filters(main_v->props.external_filter, FALSE);
 		main_v->props.external_outputbox = update_outputbox(main_v->props.external_outputbox, FALSE);
 	}
-
+#ifdef OLDTEMPLATES
 	if (main_v->props.templates == NULL) {
 		main_v->props.templates =
 			g_list_append(main_v->props.templates,
@@ -884,8 +915,15 @@ rcfile_parse_main(void)
 		main_v->props.templates =
 			g_list_append(main_v->props.templates,
 						  array_from_arglist(_("PHP"), PKGDATADIR "/templates/PHP", NULL));
+		main_v->props.templates =
+			g_list_append(main_v->props.templates,
+						  array_from_arglist(_("LaTex Book"), PKGDATADIR "/templates/Latex_Book", NULL));
+		main_v->props.templates =
+			g_list_append(main_v->props.templates,
+						  array_from_arglist(_("LaTex Presentation"), PKGDATADIR "/templates/Latex_Presentation", NULL));
 		/* TODO: list the templates in the directory */
 	}
+#endif
 }
 
 gint
@@ -893,10 +931,10 @@ rcfile_save_main(void)
 {
 	gint ret;
 	GFile *filename = user_bfdir(CURCONFIG);
-	
+
 	g_free(main_v->props.config_version);
 	main_v->props.config_version = g_strdup(VERSION);
-	
+
 	ret = save_config_file(main_configlist, filename);
 	g_object_unref(filename);
 	return ret;
@@ -973,7 +1011,6 @@ return_globalsession_configlist(gboolean init_values)
 	init_prop_integer(&config_rc, &main_v->globses.left_panel_width, "left_panel_width:", 150, init_values);
 	/*init_prop_integer   (&config_rc, &main_v->globses.lasttime_filetypes, "lasttime_filetypes:", 0, init_values);
 	   init_prop_integer   (&config_rc, &main_v->globses.lasttime_encodings, "lasttime_encodings:", 0, init_values); */
-	init_prop_integer(&config_rc, &main_v->globses.snr_select_match, "snr_select_match:", 1, init_values);
 	init_prop_integer(&config_rc, &main_v->globses.bookmarks_default_store, "bookmarks_default_store:", 1,
 					  init_values);
 	init_prop_integer(&config_rc, &main_v->globses.image_thumbnail_refresh_quality,
@@ -987,6 +1024,8 @@ return_globalsession_configlist(gboolean init_values)
 	init_prop_string(&config_rc, &main_v->globses.image_thumnailformatstring, "image_thumnailformatstring:",
 					 (init_values ?
 					  "<a href=\"%r\"><img src=\"%t\" width=\"%x\" height=\"%y\" border=\"0\"></a>" : NULL));
+	init_prop_integer(&config_rc, &main_v->globses.filter_on_selection_mode,
+					  "filter_on_selection_mode:", 0, init_values);
 	init_prop_arraylist(&config_rc, &main_v->globses.filefilters, "filefilters:", 4, init_values);
 	init_prop_arraylist(&config_rc, &main_v->globses.reference_files, "reference_files:", 2, init_values);
 	init_prop_limitedstringlist(&config_rc, &main_v->globses.recent_projects, "recent_projects:",
@@ -1017,7 +1056,7 @@ return_session_configlist(GHashTable * configlist, Tsessionvars * session)
 	init_prop_integer(&configlist, &session->view_blockstack, "view_blockstack:", 1, FALSE);
 	init_prop_integer(&configlist, &session->autocomplete, "autocomplete:", 1, FALSE);
 	init_prop_integer(&configlist, &session->show_mbhl, "show_mbhl:", 1, FALSE);
-	
+
 	init_prop_integer(&configlist, &session->snr3_type, "snr_type:", 1, FALSE);
 	init_prop_integer(&configlist, &session->snr3_replacetype, "snr_replacetype:", 1, FALSE);
 	init_prop_integer(&configlist, &session->snr3_scope, "snr_scope:", 1, FALSE);
@@ -1026,7 +1065,14 @@ return_session_configlist(GHashTable * configlist, Tsessionvars * session)
 	init_prop_integer(&configlist, &session->snr3_dotmatchall, "snr_dotmatchall:", 1, FALSE);
 	init_prop_integer(&configlist, &session->snr3_recursion_level, "snr_recursion_level:", 5, FALSE);
 
+	init_prop_integer(&configlist, &session->ssearch_regex, "ssearch_regex:", 0, FALSE);
+	init_prop_integer(&configlist, &session->ssearch_dotmatchall, "ssearch_dotmatchall:", 0, FALSE);
+	init_prop_integer(&configlist, &session->ssearch_unescape, "ssearch_unescape:", 0, FALSE);
+	init_prop_integer(&configlist, &session->ssearch_casesens, "ssearch_casesens:", 0, FALSE);
+	init_prop_string_with_escape(&configlist, &session->ssearch_text, "ssearch_text:", NULL);
+
 	init_prop_integer(&configlist, &session->display_right_margin, "display_right_margin:", 0, FALSE);
+	init_prop_integer(&configlist, &session->show_visible_spacing, "show_visible_spacing:", 0, FALSE);
 #ifdef HAVE_LIBENCHANT
 	init_prop_integer(&configlist, &session->spell_check_default, "spell_check_default:", 1, FALSE);
 	init_prop_integer(&configlist, &session->spell_insert_entities, "spell_insert_entities:", 0, FALSE);
@@ -1085,14 +1131,14 @@ return_session_configlist(GHashTable * configlist, Tsessionvars * session)
 static void
 setup_session_after_parse(Tsessionvars * session)
 {
-	if (session->editor_tab_width < 2)
+	if (session->editor_tab_width < 1)
 		session->editor_tab_width = 3;
 
 	if (session->default_mime_type == NULL)
 		session->default_mime_type = g_strdup("text/plain");
-	
+
 	if (session->filegloblist == NULL) {
-		session->filegloblist = list_from_arglist(TRUE, "*", 
+		session->filegloblist = list_from_arglist(TRUE, "*",
 				"*.c",
 				"*.cgi",
 				"*.cpp",
@@ -1110,7 +1156,7 @@ setup_session_after_parse(Tsessionvars * session)
 				"*.xml" , NULL);
 	}
 /* TODO: set spell check language to a sensible default
-	 
+
 #ifdef HAVE_LIBENCHANT
 	if (session->spell_lang==NULL)
 		session->spell_lang=
@@ -1171,6 +1217,31 @@ rcfile_save_global_session(void)
 	return retval;
 }
 
+static void
+add_new_encodings(GList *defaults, GList *current)
+{
+	GHashTable *ht;
+	GList *tmplist, *last;
+	ht = g_hash_table_new(g_str_hash, g_str_equal);
+
+	for (tmplist=g_list_first(current);tmplist;tmplist=tmplist->next) {
+		gchar **arr = tmplist->data;
+		g_hash_table_insert(ht, arr[1], GINT_TO_POINTER(1));
+		last = tmplist;
+	}
+	for (tmplist=g_list_first(defaults);tmplist;tmplist=tmplist->next) {
+		gchar **arr = tmplist->data;
+		if (g_hash_table_lookup(ht, arr[1])) {
+			g_strfreev(arr);
+		} else {
+			DEBUG_MSG("append encoding %s\n",arr[1]);
+			last = g_list_append(last, arr);
+			last = g_list_last(last);
+		}
+	}
+	g_hash_table_destroy(ht);
+}
+
 /* should be called AFTER the normal properties are loaded, becauses return_session_configlist() uses
  settings from main_v->props */
 gboolean
@@ -1200,19 +1271,89 @@ rcfile_parse_global_session(void)
 		arr = array_from_arglist(_("Images"), "1", "image/png:image/jpeg:image/gif:image/tiff", "", NULL);
 		main_v->globses.filefilters = g_list_prepend(main_v->globses.filefilters, arr);
 		arr =
-			array_from_arglist(_("Hide objectfiles"), "0", "application/octet-stream:application/x-object",
-							   "", NULL);
+			array_from_arglist(_("Hide objectfiles"), "0", "",
+							   "*.o", NULL);
 		main_v->globses.filefilters = g_list_prepend(main_v->globses.filefilters, arr);
 	}
-	if (main_v->globses.encodings == NULL) {
+	if (main_v->globses.encodings == NULL || main_v->props.rcfile_from_old_version == 1) {
 		GFile *defaultfile =
 			return_first_existing_filename(PKGDATADIR "/encodings", "data/encodings", "../data/encodings",
 										   NULL);
 		if (defaultfile) {
-			main_v->globses.encodings = get_list(defaultfile, NULL, TRUE);
+			GList *default_encodings = get_list(defaultfile, NULL, TRUE);
+			if (main_v->globses.encodings == NULL) {
+				main_v->globses.encodings = default_encodings;
+			} else {
+				add_new_encodings(default_encodings, main_v->globses.encodings);
+				g_list_free(default_encodings);
+			}
 			g_object_unref(defaultfile);
 		}
 	}
 
 	return retval;
 }
+static gint
+sort_templates(gconstpointer a,gconstpointer b)
+{
+	return g_strcmp0(((gchar **)a)[0], ((gchar **)b)[0]);
+}
+
+
+static GList *
+load_templates_from_dir(GFile *uri, gboolean create_ifnexist)
+{
+	GFileEnumerator *en;
+	GFile *child;
+	GError *gerror=NULL;
+	GFileInfo *finfo;
+	GList *retlist=NULL;
+
+
+	en = g_file_enumerate_children(uri,G_FILE_ATTRIBUTE_STANDARD_NAME,G_FILE_QUERY_INFO_NONE, NULL, &gerror);
+	if (gerror) {
+		if (gerror->code == G_IO_ERROR_NOT_FOUND && create_ifnexist) {
+			g_file_make_directory(uri,NULL,NULL);
+		} else {
+			g_print("failed to load templates: %s\n",gerror->message);
+		}
+		g_error_free(gerror);
+		return NULL;
+	}
+
+	finfo = g_file_enumerator_next_file(en,NULL,&gerror);
+	while(finfo) {
+		gchar *tmp, **arr;
+		child = g_file_get_child(uri, g_file_info_get_name(finfo));
+    	arr = g_malloc(sizeof(char *)* 3);
+    	arr[0] = g_strdup(g_file_info_get_name(finfo));
+		arr[1] = g_file_get_path(child);
+		arr[2] = NULL;
+		while ((tmp = strchr(arr[0], '_')) != NULL) {
+    		*tmp = ' ';
+    	}
+		retlist = g_list_prepend(retlist, arr);
+		g_object_unref(child);
+		g_object_unref(finfo);
+		finfo = g_file_enumerator_next_file(en,NULL,&gerror);
+	}
+	if (gerror) {
+		g_print("failed to list templates: %s\n",gerror->message);
+		g_error_free(gerror);
+	}
+	g_object_unref(en);
+	return g_list_sort(retlist, sort_templates);
+}
+
+void
+load_templates(void)
+{
+	GList *t1, *t2;
+	GFile *uri = user_bfdir("templates/");
+	t1 = load_templates_from_dir(uri, TRUE);
+	g_object_unref(uri);
+	uri = g_file_new_for_path(PKGDATADIR"/templates/");
+	t2 = load_templates_from_dir(uri, FALSE);
+	g_object_unref(uri);
+	main_v->templates = g_list_concat(t1, t2);
+}
