@@ -173,12 +173,13 @@ void snr3_run_in_doc(Tsnr3run *s3run, Tdocument *doc, gint so, gint eo, gboolean
 void snr3_run_go(Tsnr3run *s3run, gboolean forward);
 void snr3run_free(Tsnr3run *s3run);
 void snr3run_unrun(Tsnr3run *s3run);
-gpointer simple_search_run(Tbfwin *bfwin, const gchar *string);
+gpointer simple_search_run(Tbfwin *bfwin, const gchar *string, Tsnr3type type
+		, gboolean casesens, gboolean dotmatchall, gboolean unescape);
 void simple_search_next(Tbfwin *bfwin);
 void snr3_advanced_dialog(Tbfwin * bfwin, const gchar *searchstring);
 
 void snr3_run_extern_replace(Tdocument * doc, const gchar * search_pattern, Tsnr3scope scope,
 							 Tsnr3type type, gboolean is_case_sens, const gchar * replace_pattern,
-							 gboolean unescape);
+							 gboolean unescape, gboolean dotmatchall);
 
 #endif /* #define __SNR3_H_ */
